using UnityEngine;
using System.Collections;
using Umeng;

public class GameCube : MonoBehaviour {
	
	public GameObject DestroyEffectRed;
	public GameObject DestroyEffectBlue;
	public GameObject DestroyEffectYellow;
	public GameObject DestroyEffectGreen;
	public GameObject DestroyEffectOri;
	public GameObject DestroyEffectPurple;
	
	public GameObject DestroyingEffectPrefab;
	public GameObject DestroyRockPrefab;
	
	private int _xCo, _yCo;
	private int _prevCOX, _prevCOY;
	private int _lastCOX, _lastCOY;
	private int _currentPosX, _currentPosY;
	
	private bool _moving;
	private float _moveSpeed;
	
	private const float SPEED_FACTOR_STEP = 1.2f;
	private float _speedFactor = 1;
	
	private bool _falled = false;
	
	private int _type;
	
	private bool _pendingDestroy;
	private int _pendingDestroyFrameCount;
	private bool _destroying;
	private const int PENDING_DESTROY_FRAME_COUNT = 15;
	private const int PENDING_DESTROY_FRAME_COUNT_FALL = 15;
	
	private bool _pendingFall;
	private bool _falling;
	private int _pendingFallFrameCount;
	private const int PENDING_FALL_FRAME_COUNT = 3;
	
	private bool _fallingClear;
	
	private int _delayFrames = 0;
	
	private Vector3 lastPosition;
	private float destroyingShakeFactor = 0.6f;
	
	private int _sniff;
	private bool _sniffFlag;
	private float _sniffHorAmt;
	private float _sniffHorAmtMax;
	private Vector3 _sniffOriPosition;
	private Quaternion _sniffOriRotate;
	private Vector3 _sniffOriScale;
	private float _sniffRotateAmt;
	private float _sniffRotateAmttMax;
	private const int SNIFF_TYPE_NONE = 0;
	private const int SNIFF_TYPE_HOR = 1;
	private const int SNIFF_TYPE_VER = 2;
	private const int SNIFF_TYPE_SCALE = 3;
	private const int SNIFF_TYPE_HOR_SCALE = 4;
	private const int SNIFF_TYPE_VER_SCALE = 5;
	private const int SNIFF_TYPE_ROTATE = 6;
	private const int SNIFF_RATE_MAX = 10000;
	
	private bool _press;
	
	private Vector3 _touchStartPosition;
	
	public const int TOUCH_DIRECTION_NONE = 0;
	public const int TOUCH_DIRECTION_UP = 1;
	public const int TOUCH_DIRECTION_DOWN = 2;
	public const int TOUCH_DIRECTION_LEFT = 3;
	public const int TOUCH_DIRECTION_RIGHT = 4;
	
	private UILabel _label;
	
	private UISprite _foregroundSprite;
	UISprite _cubeBackgroundSprite;
	
	bool _rotation = false;

	// Use this for initialization
	void Start () {		
		_moveSpeed = GameEngine.GetInstance().GetCubeMoveSpeed();
		_pendingDestroy = false;
		_pendingFall = false;
		_falling = false;
		_destroying = false;
		_fallingClear = false;
		
		_press = false;
		
		_prevCOX = -1;
		_prevCOY = -1;
		
		_sniffOriPosition = transform.localPosition;
		
		_cubeBackgroundSprite = transform.FindChild("Background").GetComponent(typeof(UISprite)) as UISprite;
		
		//_label = transform.FindChild("Label").GetComponent(typeof(UILabel)) as UILabel;
	}
	
	// Update is called once per frame
	void Update () {
		
		//_label.text = this._type.ToString();
		
		if(GameEngine.GetInstance().IsGamePause())
		{
			return;
		}
		
		if(_xCo<0 || _yCo<0)
			return;
		
		if(_pendingDestroy)
		{
			bool chainFlag = false;
			_pendingDestroyFrameCount++;
			if(_pendingDestroyFrameCount == PENDING_DESTROY_FRAME_COUNT+_delayFrames)
			{
				chainFlag = InvokeDestroyEffect();
				//MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_DESTROY, 100);
				MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_SAND, 100, 3);
			}
			if(chainFlag)
			{
				_pendingDestroy = false;
				_pendingDestroyFrameCount = 0;
				this.transform.localPosition = lastPosition;
				return;
			}
			if(_pendingDestroyFrameCount == PENDING_DESTROY_FRAME_COUNT_FALL+_delayFrames)
			{
				NotifyDestroyed();
			}
			this.transform.localPosition = new Vector3(
				lastPosition.x+Random.Range(-5,5)*destroyingShakeFactor,
				lastPosition.y+Random.Range(-5,5)*destroyingShakeFactor,
				lastPosition.z
				);
			return;
		}
		
		if(_pendingFall)
		{
			_pendingFallFrameCount++;
			if(_pendingFallFrameCount == PENDING_FALL_FRAME_COUNT)
			{
				/*
				if(!GameEngine.GetInstance().ISDestroying(_prevCOX-1,_prevCOY) &&
					!GameEngine.GetInstance().ISStatic(_prevCOX-1,_prevCOY))
				{
					StartFall();
				}
				*/
				StartFall();
				_pendingFall = false;
			}
		}
		
		if(_rotation)
		{
			//transform.Rotate(new Vector3(0,0,1), 10*Time.deltaTime);
			_foregroundSprite.transform.Rotate(new Vector3(0,0,1), 20*Time.deltaTime);
			_foregroundSprite.MarkAsChanged();
		}
		
		if(_moving)
		{
			Vector3 targetPos;
			//Debug.Log("Cube Moving");
			targetPos = GameEngine.GetInstance().GetCubePosition(_xCo, _yCo);
			if(_fallingClear)
			{
				targetPos = new Vector3(
					targetPos.x,
					-400,
					targetPos.z
					);
			}
			if(this.transform.localPosition.x == targetPos.x &&
				this.transform.localPosition.y == targetPos.y)
			{
				_moving = false;
				_falling = false;
				if(!_fallingClear)
					GameEngine.GetInstance().CubeAtPosition(_xCo, _yCo, _type, this.gameObject);
				else
					_fallingClear = false;
				_prevCOX = _xCo;
				_prevCOY = _yCo;
				if(!_falled)
				{
					_falled = true;
					MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_FALL_GROUND, 100, 3);
				}
				return;
			}
			if(this.transform.localPosition.x != targetPos.x)
			{
				if(this.transform.localPosition.x > targetPos.x)
				{
					if((this.transform.localPosition.x - targetPos.x)<MoveSpeed)
					{
						this.transform.localPosition = new Vector3(
							targetPos.x, 
							this.transform.
							localPosition.y, 0);
					}else
					{
						this.transform.localPosition = new Vector3(
							this.transform.localPosition.x-MoveSpeed, 
							this.transform.localPosition.y, 
							0);
					}
				}else
				{
					if((targetPos.x - this.transform.localPosition.x)<MoveSpeed)
					{
						this.transform.localPosition = new Vector3(targetPos.x, this.transform.localPosition.y, 0);
					}else
					{
						this.transform.localPosition = new Vector3(
							this.transform.localPosition.x+MoveSpeed, 
							this.transform.localPosition.y, 
							0);
					}
				}
			}
			if(this.transform.localPosition.y != targetPos.y)
			{
				if(this.transform.localPosition.y > targetPos.y)
				{
					if((this.transform.localPosition.y - targetPos.y)<MoveSpeed)
					{
						this.transform.localPosition = new Vector3(this.transform.localPosition.x, targetPos.y, 0);
					}else
					{
						this.transform.localPosition = new Vector3(
							this.transform.localPosition.x, 
							this.transform.localPosition.y-MoveSpeed, 
							0);
					}
				}else
				{
					if((targetPos.y - this.transform.localPosition.y)<MoveSpeed)
					{
						this.transform.localPosition = new Vector3(this.transform.localPosition.x, targetPos.y, 0);
					}else
					{
						this.transform.localPosition = new Vector3(
							this.transform.localPosition.x, 
							this.transform.localPosition.y+MoveSpeed, 
							0);
					}
				}
			}
		}
		
		//CheckSniff();
	}
	
	private void CheckSniff()
	{
		//return;
		
		if(_pendingFall || _pendingDestroy || _moving)
			return;
		
		if(_sniff > SNIFF_TYPE_NONE)
		{
			switch(_sniff)
			{
			case SNIFF_TYPE_HOR:
				if(_sniffFlag)
				{
					_sniffHorAmt++;
					if(_sniffHorAmt>_sniffHorAmtMax)
						_sniffFlag = !_sniffFlag;
					
				}else
				{
					_sniffHorAmt--;
					if(_sniffHorAmt<-_sniffHorAmtMax)
						_sniffFlag = !_sniffFlag;
				}
				transform.localPosition = new Vector3(
					_sniffOriPosition.x+_sniffHorAmt,
					_sniffOriPosition.y,
					_sniffOriPosition.z
					);
				if(_sniffHorAmt==0)
				{
					if(Random.Range(0, 4)==1)
					{
						StopSniff();
					}
				}
				break;
			case SNIFF_TYPE_VER:
				if(_sniffFlag)
				{
					_sniffHorAmt++;
					if(_sniffHorAmt>_sniffHorAmtMax)
						_sniffFlag = !_sniffFlag;
					
				}else
				{
					_sniffHorAmt--;
					if(_sniffHorAmt<-_sniffHorAmtMax)
						_sniffFlag = !_sniffFlag;
				}
				transform.localPosition = new Vector3(
					_sniffOriPosition.x,
					_sniffOriPosition.y+_sniffHorAmt,
					_sniffOriPosition.z
					);
				if(_sniffHorAmt==0)
				{
					if(Random.Range(0, 4)==1)
					{
						StopSniff();
					}
				}
				break;
			case SNIFF_TYPE_SCALE:
				break;
			case SNIFF_TYPE_HOR_SCALE:
				break;
			case SNIFF_TYPE_VER_SCALE:
				break;
			case SNIFF_TYPE_ROTATE:
				if(_sniffFlag)
				{
					_sniffHorAmt += 0.05f;
					if(_sniffHorAmt>_sniffHorAmtMax)
						_sniffFlag = !_sniffFlag;
					
				}else
				{
					_sniffHorAmt -= 0.05f;
					if(_sniffHorAmt<-_sniffHorAmtMax)
						_sniffFlag = !_sniffFlag;
				}
				transform.transform.localRotation = new Quaternion(
					_sniffOriRotate.x,
					_sniffOriRotate.y,
					_sniffOriRotate.z+_sniffHorAmt,
					_sniffOriRotate.w
					);
				if(_sniffHorAmt<0.05f && _sniffHorAmt>-0.05f)
				{
					if(Random.Range(0, 4)==1)
					{
						StopSniff();
					}
				}
				break;
			}
		}else
		{
			if(Random.Range(0, SNIFF_RATE_MAX)==_type)
			{
				//Debug.Log("Sniff start");
				_sniff = Random.Range(SNIFF_TYPE_NONE, SNIFF_TYPE_ROTATE+1);
				switch(_sniff)
				{
					case SNIFF_TYPE_HOR:
						_sniff = SNIFF_TYPE_HOR;
						_sniffHorAmt = 0;
						_sniffFlag = true;
						_sniffHorAmtMax = 4;
						_sniffOriPosition = transform.localPosition;
						break;
					case SNIFF_TYPE_VER:
						_sniff = SNIFF_TYPE_VER;
						_sniffHorAmt = 0;
						_sniffFlag = true;
						_sniffHorAmtMax = 4;
						_sniffOriPosition = transform.localPosition;
						break;
					case SNIFF_TYPE_SCALE:
						break;
					case SNIFF_TYPE_HOR_SCALE:
						break;
					case SNIFF_TYPE_VER_SCALE:
						break;
					case SNIFF_TYPE_ROTATE:
						_sniff = SNIFF_TYPE_ROTATE;
						_sniffHorAmt = 0;
						_sniffFlag = true;
						_sniffHorAmtMax = 4f;
						_sniffOriRotate = transform.localRotation;
						break;
				}
			}
		}
	}
	
	public void StopSniff()
	{
		return;
		//Debug.Log("Sniff stop");
		switch(_sniff)
		{
			case SNIFF_TYPE_HOR:
				//Debug.Log("Sniff stop  revert HOR");
				this.transform.localPosition = _sniffOriPosition;
				break;
			case SNIFF_TYPE_VER:
				//Debug.Log("Sniff stop  revert VER");
				this.transform.localPosition = _sniffOriPosition;
				break;
			case SNIFF_TYPE_SCALE:
				//this.transform.localScale = _sniffOriScale;
				break;
			case SNIFF_TYPE_HOR_SCALE:
				//this.transform.localScale = _sniffOriScale;
				break;
			case SNIFF_TYPE_VER_SCALE:
				//this.transform.localScale = _sniffOriScale;
				break;
			case SNIFF_TYPE_ROTATE:
				this.transform.localRotation = _sniffOriRotate;
				break;
		}
		_sniff = SNIFF_TYPE_NONE;
	}
	
	void OnClick()
	{
		//DestroyMe();
	}
	
	void OnPress(bool press)
	{
		if(this._moving)
			return;
		
		if(GameEngine.GetInstance().ChuiMode)
		{
			if(!press)
			{
				if(GameEngine.GetInstance().ChuiX==_xCo && GameEngine.GetInstance().ChuiY==_yCo)
				{
					DestroyByChui();
					GameEngine.GetInstance().ChuiMode = false;
				}else
				{
					ChuiScript.GetInstance().transform.localPosition = new Vector3(
						this.transform.localPosition.x,
						this.transform.localPosition.y,
						-2
						);
					GameEngine.GetInstance().ChuiX = _xCo;
					GameEngine.GetInstance().ChuiY = _yCo;
				}
			}
			return;
		}
		
		//_press = true;
		if(GameEngine.GetInstance().NotifyClickCube(_xCo,_yCo,this.gameObject))
			return;
		#if UNITY_EDITOR
		if(press)
		{
			_press = true;
			_touchStartPosition = Input.mousePosition;
		}else
		{
			if(_press)
			{
				int direction = CheckDirection(_touchStartPosition, Input.mousePosition);
				if(!_pendingDestroy)
				{
					GameEngine.GetInstance().NotifyCubeDragged(_xCo,_yCo, direction, this.gameObject);
				}
			}
			_press = false;
		}
#elif UNITY_IPHONE || UNITY_ANDROID
		if(press)
		{
			//Debug.Log("OnPress  enter");
			_press = true;
			_touchStartPosition = Input.GetTouch(0).position;
		}else
		{
			if(_press)
			{
				//Debug.Log("OnRelease  enter");
				int direction = CheckDirection(_touchStartPosition, Input.GetTouch(0).position);
				if(!_pendingDestroy)
				{
					GameEngine.GetInstance().NotifyCubeDragged(_xCo,_yCo, direction, this.gameObject);
				}
			}
			_press = false;
		}
#endif
	}
	
	private int CheckDirection(Vector3 start, Vector3 end)
	{
		Vector3 diff;
		diff = end - start;
		
		if(diff.sqrMagnitude < 5)
			return TOUCH_DIRECTION_NONE;
		
		if(Mathf.Abs(diff.x)>Mathf.Abs(diff.y))
		{
			/**
			 * left or right
			 * */
			if(diff.x>0)
			{
				return TOUCH_DIRECTION_RIGHT;
			}else
			{
				return TOUCH_DIRECTION_LEFT;
			}
		}else
		{
			/**
			 * up or down.
			 * */
			if(diff.y>0)
			{
				return TOUCH_DIRECTION_UP;
			}
			else
			{
				return TOUCH_DIRECTION_DOWN;
			}
		}
	}
	
	private void DestroyByChui()
	{
		UserManager.GetInstance().UseCorns(UserManager.GetInstance().CHUI_COST_CORNS);
		GA.Use("pChui", 1, 300);
		if(StageManager.GetInstance().BackgroundBlockChain(_xCo,_yCo))
		{
			Debug.Log("DestroyByChui   chain1");
			DisableChain();
			StageManager.GetInstance().SetBackgroundBlank(_xCo,_yCo);
			GameEngine.GetInstance().NotifyCubeNChained(_xCo, _yCo, _type, this.gameObject);
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_CHAIN_OUT, 100, 2);
			return ;
		}
		
		if(StageManager.GetInstance().BackgroundBlockDChain(_xCo,_yCo))
		{
			Debug.Log("DestroyByChui   d chain");
			EnableChain1();
			StageManager.GetInstance().SetBackgroundChain1(_xCo,_yCo);
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_CHAIN_OUT, 100, 2);
			return ;
		}
		
		if(StageManager.GetInstance().BackgroundBlockTChain(_xCo,_yCo))
		{
			Debug.Log("DestroyByChui   T chain");
			EnableChain2();
			StageManager.GetInstance().SetBackgroundChain2(_xCo,_yCo);
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ROCK, 100, 2);
			
			{
				GameObject rockP = Instantiate(DestroyRockPrefab) as GameObject;
				rockP.transform.position = new Vector3(
					this.transform.position.x,
					this.transform.position.y,
					this.transform.position.z-0.1f
					);
			}
			
			return ;
		}
		
		GameObject obj = null;
		switch(_type)
		{
		case 1:
			obj = Instantiate(DestroyEffectRed) as GameObject;
			break;
		case 2:
			obj = Instantiate(DestroyEffectPurple) as GameObject;
			break;
		case 3:
			obj = Instantiate(DestroyEffectGreen) as GameObject;
			break;
		case 4:
			obj = Instantiate(DestroyEffectYellow) as GameObject;
			break;
		case 5:
			obj = Instantiate(DestroyEffectBlue) as GameObject;
			break;
		case 6:
			obj = Instantiate(DestroyEffectOri) as GameObject;
			break;
		case 7:
			obj = Instantiate(DestroyEffectOri) as GameObject;
			break;
		}
		
		obj.transform.position = new Vector3(
			this.transform.position.x,
			this.transform.position.y,
			this.transform.position.z-0.1f
			);
		InvokeCubeAttack();
		
		SetCubeType(0);
		MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_DESTROY, 100);
		GameEngine.GetInstance().NotifyCubeDestroyed(_xCo,_yCo, this.gameObject, _xCo, _yCo);
		Destroy(this.gameObject);
	}
	
	private void NotifyDestroyed()
	{
		GameEngine.GetInstance().NotifyCubeDestroyed(_xCo,_yCo, this.gameObject, _lastCOX, _lastCOY);
		Destroy(this.gameObject);
	}
	
	private bool InvokeDestroyEffect(bool destroying=false)
	{
		GameObject obj = null;
		
		if(destroying)
		{
			obj = Instantiate(DestroyingEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				this.transform.position.x,
				this.transform.position.y,
				this.transform.position.z-0.1f
				);
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_DESTROYING, 10);
			return false;
		}
		
		if(StageManager.GetInstance().BackgroundBlockChain(_xCo,_yCo))
		{
			Debug.Log("InvokeDestroyEffect   chain1");
			DisableChain();
			StageManager.GetInstance().SetBackgroundBlank(_xCo,_yCo);
			GameEngine.GetInstance().NotifyCubeNChained(_xCo, _yCo, _type, this.gameObject);
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_CHAIN_OUT, 100, 2);
			return true;
		}
		
		if(StageManager.GetInstance().BackgroundBlockDChain(_xCo,_yCo))
		{
			Debug.Log("InvokeDestroyEffect   d chain");
			EnableChain1();
			StageManager.GetInstance().SetBackgroundChain1(_xCo,_yCo);
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_CHAIN_OUT, 100, 2);
			return true;
		}
		
		if(StageManager.GetInstance().BackgroundBlockTChain(_xCo,_yCo))
		{
			Debug.Log("DestroyByChui   T chain");
			EnableChain2();
			StageManager.GetInstance().SetBackgroundChain2(_xCo,_yCo);
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ROCK, 100, 2);
			
			{
				GameObject rockP = Instantiate(DestroyRockPrefab) as GameObject;
				rockP.transform.position = new Vector3(
					this.transform.position.x,
					this.transform.position.y,
					this.transform.position.z-0.1f
					);
			}
			
			return true;
		}
		
		switch(_type)
		{
		case 1:
			obj = Instantiate(DestroyEffectRed) as GameObject;
			break;
		case 2:
			obj = Instantiate(DestroyEffectPurple) as GameObject;
			break;
		case 3:
			obj = Instantiate(DestroyEffectGreen) as GameObject;
			break;
		case 4:
			obj = Instantiate(DestroyEffectYellow) as GameObject;
			break;
		case 5:
			obj = Instantiate(DestroyEffectBlue) as GameObject;
			break;
		case 6:
			obj = Instantiate(DestroyEffectOri) as GameObject;
			break;
		case 7:
			obj = Instantiate(DestroyEffectOri) as GameObject;
			break;
		}
		//Debug.Log("InvokeDestroyEffect type="+_type);
		
		obj.transform.position = new Vector3(
			this.transform.position.x,
			this.transform.position.y,
			this.transform.position.z-0.1f
			);
		InvokeCubeAttack();
		
		SetCubeType(0);
		
		GameEngine.GetInstance().NotifyDestroying(_xCo,_yCo,obj);
		
		return false;
		
	}
	
	private void InvokeCubeAttack()
	{
		GameObject obj = null;
		
		//Debug.Log("InvokeCubeAttack  type="+_type);
		if(_type == 5)
		{
			obj = Instantiate(GameEngine.GetInstance().cubeBuletFire) as GameObject;
		}else
		{
			return;
		}
		obj.transform.parent = UIManager.GetInstance().GetUICamera().transform;
		obj.transform.localScale = new Vector3(1,1,1);
		obj.transform.position = new Vector3(
			this.transform.position.x,
			this.transform.position.y,
			this.transform.position.z-0.1f
			);
		
		ScoreBulet scoreScript;
		
		scoreScript = obj.GetComponent(typeof(ScoreBulet)) as ScoreBulet;
		scoreScript.SetScore(1);
		/*
		GameObject obj = null;
		switch(_type)
		{
		case 1:
			obj = Instantiate(GameEngine.GetInstance().cubeBuletFire) as GameObject;
			break;
		case 2:
			obj = Instantiate(GameEngine.GetInstance().cubeBuletWater) as GameObject;
			break;
		case 3:
			obj = Instantiate(GameEngine.GetInstance().cubeBuletMetal) as GameObject;
			break;
		case 4:
			obj = Instantiate(GameEngine.GetInstance().cubeBuletWood) as GameObject;
			break;
		case 5:
			obj = Instantiate(GameEngine.GetInstance().cubeBuletStone) as GameObject;
			break;
		case 6:
			obj = Instantiate(GameEngine.GetInstance().cubeBuletBlack) as GameObject;
			break;
		case 7:
			obj = Instantiate(GameEngine.GetInstance().cubeBuletLight) as GameObject;
			break;
		}
		obj.transform.parent = UIManager.GetInstance().GetUICamera().transform;
		obj.transform.localScale = new Vector3(1,1,1);
		obj.transform.position = new Vector3(
			this.transform.position.x,
			this.transform.position.y,
			this.transform.position.z-0.1f
			);
		
		ScoreBulet scoreScript;
		
		scoreScript = obj.GetComponent(typeof(ScoreBulet)) as ScoreBulet;
		scoreScript.SetScore(10*TotemPanel.GetInstance().GetScoreFactor()*
			GameEngine.GetInstance().GetMagicFactor());
			*/
	}
	
	public void SetOrignalPosition(Vector3 v)
	{
		_moving = true;
		_speedFactor = SPEED_FACTOR_STEP;
	}
	
	public void SetGameCoordinate(int x, int y)
	{
		_xCo = x;
		_yCo = y;
	}
	
	public void StartMove(float fast=1.0f)
	{
		StopSniff();
		_moving = true;
		_speedFactor = SPEED_FACTOR_STEP*fast;
	}
	
	public void StartFall()
	{
		_moving = true;
		_falling = true;
		_speedFactor = SPEED_FACTOR_STEP;
		GameEngine.GetInstance().NotifyFalling(_prevCOX,_prevCOY,this.gameObject);
	}
	
	public float MoveSpeed
	{
		get{
			float res;
			if(_fallingClear)
			{
				res = _moveSpeed*_speedFactor*Time.deltaTime/10;
			}else
			{
				res = _moveSpeed*_speedFactor*Time.deltaTime;
			}
			_speedFactor *= SPEED_FACTOR_STEP;
			return res;
		}
	}
	
	public void FallNLevel(int n)
	{
		_moveSpeed = GameEngine.GetInstance().GetCubeFallSpeed();
		//GameEngine.GetInstance().NotifyFalling(_xCo,_yCo,this.gameObject);
		_xCo -= n;
		_pendingFall = true;
		_pendingFallFrameCount = 0;
		
		StopSniff();
	}
	
	public void FallOneLevel()
	{
		/*
		if(_pendingDestroy || _destroying)
		{
			Debug.Log("["+_xCo+","+_yCo+"]  fall one level  NOT do");
			return;
		}
		*/
		//Debug.Log("["+_xCo+","+_yCo+"] FallOneLevel");
		_moveSpeed = GameEngine.GetInstance().GetCubeFallSpeed();
		//GameEngine.GetInstance().NotifyFalling(_xCo,_yCo,this.gameObject);
		_xCo--;
		_pendingFall = true;
		_pendingFallFrameCount = 0;
		
		StopSniff();
	}
	
	public int GetCubeType()
	{
		return _type;
	}
	
	public void SetCubeType(int type)
	{
		GameObject obj;
		_type = type;
		UIImageButton sprite;
		UISprite sprite1;
		
		//Debug.Log("Type = "+type);
		
		obj = this.transform.FindChild("Background").gameObject;
		sprite = this.gameObject.GetComponent(typeof(UIImageButton)) as UIImageButton;
		sprite1 = obj.GetComponent(typeof(UISprite)) as UISprite;
		/**
		 * 1: fireBall
		 * 2: waterBall
		 * 3: metalBall
		 * 4: woodBall
		 * 5: stoneBall
		 * 6: blackBall
		 * 7: lightBall
		 * */
		switch(_type)
		{
		case 1:
			sprite.normalSprite = "Cube1";
			sprite.hoverSprite = "Cube1";
			sprite.pressedSprite = "Cube1";
			sprite1.spriteName = "Cube1";
			break;
		case 2:
			sprite.normalSprite = "Cube2";
			sprite.hoverSprite = "Cube2";
			sprite.pressedSprite = "Cube2";
			sprite1.spriteName = "Cube2";
			break;
		case 3:
			sprite.normalSprite = "Cube3";
			sprite.hoverSprite = "Cube3";
			sprite.pressedSprite = "Cube3";
			sprite1.spriteName = "Cube3";
			break;
		case 4:
			sprite.normalSprite = "Cube4";
			sprite.hoverSprite = "Cube4";
			sprite.pressedSprite = "Cube4";
			sprite1.spriteName = "Cube4";
			break;
		case 5:
			sprite.normalSprite = "Cube5";
			sprite.hoverSprite = "Cube5";
			sprite.pressedSprite = "Cube5";
			sprite1.spriteName = "Cube5";
			break;
		case 6:
			sprite.normalSprite = "Cube6";
			sprite.hoverSprite = "Cube6";
			sprite.pressedSprite = "Cube6";
			sprite1.spriteName = "Cube6";
			break;
		case 7:
			sprite.normalSprite = "Cube7";
			sprite.hoverSprite = "Cube7";
			sprite.pressedSprite = "Cube7";
			sprite1.spriteName = "Cube7";
			break;
		default:
			Destroy(sprite1);
			break;
		}
		
	}
	
	int _chainType = CHAIN_TYPE_NONE;
	const int CHAIN_TYPE_NONE = 0;
	const int CHAIN_TYPE_1 = 1;
	const int CHAIN_TYPE_2 = 2;
	const int CHAIN_TYPE_3 = 3;
	public void EnableChain1()
	{
		_foregroundSprite = transform.FindChild("ForeGround").GetComponent(typeof(UISprite)) as UISprite;
		_foregroundSprite.active = true;
		_foregroundSprite.alpha = 1;
		_foregroundSprite.spriteName = "Chain1";
		_rotation = false;
		_chainType = CHAIN_TYPE_1;
	}
	
	public void EnableChain2()
	{
		_foregroundSprite = transform.FindChild("ForeGround").GetComponent(typeof(UISprite)) as UISprite;
		_foregroundSprite.active = true;
		_foregroundSprite.alpha = 1;
		_foregroundSprite.spriteName = "Chain2";
		_rotation = false;
		_chainType = CHAIN_TYPE_2;
	}
	
	public void EnableChain3()
	{
		_foregroundSprite = transform.FindChild("ForeGround").GetComponent(typeof(UISprite)) as UISprite;
		_foregroundSprite.active = true;
		_foregroundSprite.alpha = 1;
		_foregroundSprite.spriteName = "Chain3";
		_rotation = true;
		_chainType = CHAIN_TYPE_3;
	}
	
	public void DisableChain()
	{
		_foregroundSprite = transform.FindChild("ForeGround").GetComponent(typeof(UISprite)) as UISprite;
		_foregroundSprite.active = false;
		_foregroundSprite.alpha = 0;
		_chainType = CHAIN_TYPE_NONE;
	}
	
	public const float REINIT_TWEEN_TIME = 0.3f;
	public void ReinitTypes()
	{
		if(_chainType != CHAIN_TYPE_NONE)
			return;
		
		iTween.RotateTo(
			_cubeBackgroundSprite.gameObject,
			iTween.Hash(
					"rotation",
					new Vector3(0,180,0),
					"time",
					REINIT_TWEEN_TIME,
					"islocal",
					true,
					"easetype",
					iTween.EaseType.linear,
					"onComplete",
					"ReinitRotateTweenComplete1",
					"onCompleteTarget",
					this.gameObject
					)
				);
	}
	void ReinitRotateTweenComplete1()
	{
		{
			int reType;
			reType = Random.Range(1, GameEngine.GetInstance()._cubeTypeCount+1);
			SetCubeType(reType);
			//GameEngine.GetInstance().NotifyCubeTypeChangedNoCheck(GetCOX(), GetCOY(), reType);
		}
		iTween.RotateTo(
			_cubeBackgroundSprite.gameObject,
			iTween.Hash(
					"rotation",
					new Vector3(0,0,0),
					"time",
					REINIT_TWEEN_TIME,
					"islocal",
					true,
					"easetype",
					iTween.EaseType.linear,
					"onComplete",
					"ReinitRotateTweenComplete2",
					"onCompleteTarget",
					this.gameObject
					)
				);
	}
	void ReinitRotateTweenComplete2()
	{
	}
	
	public void DestroyMeIm()
	{
		if(this.gameObject != null)
			Destroy(this.gameObject);
	}
	
	public void DestroyMe(int delayFrames)
	{
		
		/*
		GameObject obj = Instantiate(DestroyEffectRed) as GameObject;
		obj.transform.position = this.transform.position;
		Destroy(this.gameObject);
		*/
		
		StopSniff();
		
		if(_pendingDestroy)
		{
			if(delayFrames>_delayFrames)
				_delayFrames = delayFrames;
			return;
		}
		//Debug.Log("Destroy ["+_xCo+","+_yCo+"]  delay="+delayFrames);
		
		_delayFrames = delayFrames;
		
		_lastCOX = _xCo;
		_lastCOY = _yCo;
		
		_pendingDestroy = true;
		lastPosition = this.transform.localPosition;
		_pendingDestroyFrameCount = 0;
		InvokeDestroyEffect(true);
		
		//GameEngine.GetInstance().NotifyDestroying(_xCo, _yCo);
	}
	
	public void FallClear()
	{
		_fallingClear = true;
		_moving = true;
	}
	
	public int GetCOX()
	{
		return _xCo;
	}
	
	public int GetCOY()
	{
		return _yCo;
	}
	
	public bool IsMoving()
	{
		return _moving;
	}
	
	public bool IsDestroying()
	{
		return _pendingDestroy;
	}
	
	public bool IsFalling()
	{
		return _falling || _pendingFall;
	}
	
	public int GetPrevX()
	{
		return _prevCOX;
	}
	
	public int GetPrevY()
	{
		return _prevCOY;
	}
}