using UnityEngine;
using System.Collections;
using Umeng;

public class GameEngine : MonoBehaviour {
	
	public int colum;
	public int row;
	public float tileWidth;
	public float tileHeight;
	public int _cubeTypeCount;
	public float _cubeMoveSpeed;
	public GameObject cubePrefab;
	public GameObject cubeFocusPrefab;
	
	public GameObject cubeBackgroundPrefab;
	private bool testCube = false;
	public GameObject cubeTestPrefab;
	
	public GameObject cubeBuletFire;
	public GameObject cubeBuletWater;
	public GameObject cubeBuletStone;
	public GameObject cubeBuletMetal;
	public GameObject cubeBuletWood;
	public GameObject cubeBuletBlack;
	public GameObject cubeBuletLight;
	
	public GameObject fireAttackEffectPrefab;
	public GameObject waterAttackEffectPrefab;
	public GameObject woodHealEffectPrefab;
	public GameObject metalAttackEffectPrefab;
	public GameObject stoneAttackEffectPrefab;
	public GameObject blackAttackEffectPrefab;
	public GameObject lightAttackEffectPrefab;
	
	public GameObject magicColumPrefab;
	public GameObject magicRowPrefab;
	public GameObject magicSquarePrefab;
	public GameObject magicAllPrefab;
	public GameObject magicTimePrefab;
	public GameObject magicScoreFactorPrefab;
	
	public bool timeMode = false;
	public int timeLimit = 100;
	
	public bool SJQMode = true;
	public GameObject DestroyWallEffectPrefab;
	public GameObject DestroyIceEffectPrefab;
	
	public UIPanel _background1Panel;
	
	private System.DateTime backupPauseTime;
	
	private static GameEngine _instance;
	
	private UIPanel _gamePanel;
	private UIPanel _backgroundPanel;
	private const int GAME_CUBE_SWITCHING = -2;
	private const int GAME_CUBE_DESTROYING = -1;
	public const int GAME_CUBE_NONE = 0;
	public const int GAME_CUBE_TYPE1 = 1;
	public const int GAME_CUBE_TYPE2 = 2;
	public const int GAME_CUBE_TYPE3 = 3;
	public const int GAME_CUBE_TYPE4 = 4;
	public const int GAME_CUBE_TYPE5 = 5;
	public const int GAME_CUBE_TYPE6 = 6;
	public const int GAME_CUBE_TYPE7 = 7;
	public const int GAME_CUBE_FIRE = 1;
	public const int GAME_CUBE_WATER = 2;
	public const int GAME_CUBE_METAL = 3;
	public const int GAME_CUBE_WOOD = 4;
	public const int GAME_CUBE_STONE = 5;
	public const int GAME_CUBE_BLACK = 6;
	public const int GAME_CUBE_LIGHT = 7;
	
	private const int BIRTH_PLACE_OFFSET = 800;
	
	private const int CUBE_DAMAGE = 15;
	
	private int[,] _data;
	private GameObject[,] _cubeData;
	private GameObject[,] _backgroundBlock;
	
	private int _moveCubeX, _moveCubeY;
	private bool _moveCubeFlag = false;
	
	private int _gameState;
	private const int GAME_STATE_NONE = 0;
	private const int GAME_STATE_INIT = 1;
	private const int GAME_STATE_READY = 2;
	private const int GAME_STATE_DISTROYING = 3;
	private const int GAME_STATE_OVER = 4;
	
	private int _gameInitIndexX;
	private int _gameInitIndexY;
	private int _gameInitFrameIndex;
	private const int GAME_INIT_FRMAE_GAP = 2;
	
	/// <summary>
	/// The birth queue.
	/// </summary>
	private Queue birthQueue;
	private int birthQueueIndex;
	
	private ArrayList fallingCubes;
	private ArrayList allCubes;
	
	private int _pressedCubeX, _pressedCubeY;
	private int _draggedCubeX, _draggedCubeY;
	private bool _switching;
	private int _switchingCube1X, _switchingCube1Y;
	private int _switchingCube2X, _switchingCube2Y;
	private int _switchingCount;
	private bool _switchingPass;
	
	private bool _gamePause;
	private bool _gamePlaying;
	private bool _gameClear = false;
	/**
	 * time bonus
	 * */
	private bool _timeBonus = false;
	private int _timeBonusStep = 1;
	private int _timeBonusFrameGap = 2;
	private int _timeBonusValue = 0;
	private int _timeFrameIndex = 0;
	
	private bool _gamePendingV = false;
	private int _gameClearIndex = 0;
	private const int GAME_CLEAR_GAP = 2;
	private const int GAME_CLEAR_FRAME_COUNTS = 200;
	
	private GameObject _cubeBackground;
	
	private bool _pendingGameLose;
	private bool _pendingGameVictory;
	private int _pendingFrameCount;
	private const int GAME_OVER_PENDING_FRAMES = 30;
	
	private System.DateTime lastUpdateTime;
	
	private System.DateTime _gameStartTime;
	
	private int magicFactor = 1;
	
	private int hintX, hintY, hintDirection;
	public const int HINT_DRECTION_LEFT = 1;
	public const int HINT_DRECTION_RIGHT = 2;
	public const int HINT_DRECTION_UP = 3;
	public const int HINT_DRECTION_DOWN = 4;

	bool escPressed = false;
	void OnGUI()
	{
		#if UNITY_ANDROID
		if(Input.GetKeyDown(KeyCode.Escape))
		{
			escPressed = true;
		}
		if(Input.GetKeyUp(KeyCode.Escape) && escPressed)
		{
			escPressed = false;
			if(MessageWindow.GetInstance().IsMessageWindowOn())
			{
				MessageWindow.GetInstance().Hide();
				return;
			}
			switch(UIManager.GetInstance().CurrentView())
			{
			case UIManager.VIEW_GAME:
				GameView.GetInstance().GamePause();
				return;
			case UIManager.VIEW_ABOUT:
			case UIManager.VIEW_CHAPTER:
			case UIManager.VIEW_CHART:
			case UIManager.VIEW_STORE:
				UIManager.GetInstance().ShowView(UIManager.VIEW_MAIN);
				return;
			case UIManager.VIEW_MAIN:
					MessageWindow.GetInstance().ShowMessage(
						"˳Ϸ",
						"ȷҪ˳Ϸ",
						MessageWindow.MESSAGE_WINDOW_QUIT_GAME
						);
				return;
			}
		}
		#endif
	}

	// Use this for initialization
	void Start () {
		_instance = this;
		
		_pendingGameLose = false;
		_pendingGameVictory = false;
		_pendingFrameCount = 0;
		
		GameObject anchor = this.transform.FindChild("Anchor").gameObject;
		_gamePanel = anchor.transform.FindChild("GamePanel").GetComponent(typeof(UIPanel)) as UIPanel;
		_backgroundPanel = anchor.transform.FindChild("BackgroundPanel").GetComponent(typeof(UIPanel)) as UIPanel;
		
		if(_cubeTypeCount>7)
		{
			_cubeTypeCount = 7;
		}
		
		ToState(GAME_STATE_NONE);
		
		lastUpdateTime = System.DateTime.Now;
		
		Application.targetFrameRate = 600;
		
		if(_background1Panel != null)
		{
			_background1Panel.gameObject.active = false;
		}
	}
	
	public void GameClear()
	{
		_gameClearIndex = 0;
		_gameClear = true;
	}
	

	bool _pausedByInterstitial = false;
	bool _pendingDisplayInterstitial = false;
	int _pendingDisplayInterstitialIndex = 0;
	const int INTERSTITIAL_GAP_FRAMES = 20;
	private void CheckInterstitial()
	{
		_pendingDisplayInterstitialIndex++;
		if(_pendingDisplayInterstitialIndex<=INTERSTITIAL_GAP_FRAMES)
		{
			return;
		}
		_pendingDisplayInterstitial = false;
#if UNITY_IPHONE
			if(!AdBannerManager.IsAdDisabled())
			{
				AdBannerManager.GetInstance().ShowInterstitialAD();
			}
#endif
	}
	public bool PausedByInterstitial
	{
		get{
		
			return _pausedByInterstitial;
		}
		/*
		set
		{
			_pausedByInterstitial = value;
		}
		*/
	}
	void OnApplicationPause(bool pause)
	{
		//Debug.Log("@@@@@@@@@@@@@@@@@@@@@@@@@@@@  OnApplicationPause  GameENGINE  enter  pause="+pause);
		if(pause)
		{
		}else
		{
			_pendingDisplayInterstitial = true;
			_pendingDisplayInterstitialIndex = 0;
		}
	}
	bool _trainedFlag = false;
	bool _trainedFlag2 = false;
	// Update is called once per frame
	void Update () {
		
		/*
		{
			System.DateTime now = System.DateTime.Now;
			double fps = 1000/now.Subtract(lastUpdateTime).TotalMilliseconds;
			lastUpdateTime = now;
			if(GameMessage.GetInstance()!=null)
				GameMessage.GetInstance().ShowMessage("fps: "+fps.ToString("00.00"));
		}
		*/
		
		if(_timeBonus)
		{
			_timeFrameIndex++;
			if(_timeFrameIndex%_timeBonusFrameGap == 0)
			{
				//Debug.Log("Time bonus   value="+_timeBonusValue);
				if(_timeBonusValue>0)
				{
					GameClock.GetInstance().UpdateTimeEx(timeLimit*1000-_timeBonusValue*1000, timeLimit);
					if(_timeBonusValue%_timeBonusStep == 0)
					{
						if(_timeBonusValue%10 == 0)
							MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_CORNS, 100,2);
						UserManager.GetInstance().AddCorns(1);
					}
					_timeBonusValue--;
				}else
				{
					_timeBonus = false;
					GameView.GetInstance().NotifyGameClearFinish();
				}
			}
			return;
		}
		
		if(_gameClear)
		{
			GameObject cube;
			GameCube cubeScript;
			if(_gameClearIndex%GAME_CLEAR_GAP==0)
			{
				if((_gameClearIndex/GAME_CLEAR_GAP)<allCubes.Count)
				{
					cube = allCubes[_gameClearIndex/GAME_CLEAR_GAP] as GameObject;
					cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
					cubeScript.FallClear();
				}
			}
			_gameClearIndex++;
			if(_gameClearIndex>GAME_CLEAR_FRAME_COUNTS)
			{
				_gameClear = false;
				//GameView.GetInstance().NotifyGameClearFinish();
				_timeBonus = true;
				_timeFrameIndex = 0;
			}
			return;
		}
		
		
		if(_pendingGameLose)
		{
			if(timeMode)
			{
			}else
			{
				//Debug.Log("Pending lose frame="+_pendingFrameCount);
				_pendingFrameCount++;
				if(_pendingFrameCount>GAME_OVER_PENDING_FRAMES)
				{
					_pendingGameLose = false;
					//GameEngine.GetInstance().GameDestroy();
					GameView.GetInstance().StageLose();
				}
			}
			return;
		}
		
		if(_pendingGameVictory)
		{
			//Debug.Log("Pending victory frame="+_pendingFrameCount);
			_pendingFrameCount++;
			if(_pendingFrameCount>GAME_OVER_PENDING_FRAMES)
			{
				_pendingGameVictory = false;
				
				//GameEngine.GetInstance().GameDestroy();
				//GameView.GetInstance().StageVictory();
				GameView.GetInstance().StageVictory();
				_gamePlaying = false;
			}
			return;
		}
		
		if(_gamePause)
			return;
		
		if(_gamePlaying && !_gamePause && !_timeBonus && !_pendingGameVictory){
			System.DateTime now = System.DateTime.Now;
			double mSecs = now.Subtract(_gameStartTime).TotalMilliseconds;
			GameClock.GetInstance().UpdateTime(mSecs, timeLimit);
			if(mSecs > timeLimit*1000)
			{
				if(_pendingGameLose || _pendingGameVictory)
				{
				}else
				{
					_pendingGameLose = true;
					_pendingFrameCount = 0;
					GameView.GetInstance().TimeUp();
					_gamePlaying = false;
				}
			}
		}
			
		if(InState(GAME_STATE_INIT))
		{
			if((_gameInitFrameIndex%GAME_INIT_FRMAE_GAP)==0)
			{
				GameObject cube;
				GameCube cubeScript;
				int cubeType;
				/**
				 * birth a cube
				 * */
				if(true)
				{
					for(int i=0; i<colum; i++)
					{
						if(!StageManager.GetInstance().BackgroundBlockWall(_gameInitIndexX, _gameInitIndexY))
						{
							//cubeType = GenerateRandomType();
							cubeType = GenerateRandomTypeEx(_gameInitIndexX, _gameInitIndexY);
							if(UserManager.GetInstance().Stage == 37)
							{
								//cubeType = stage37[_gameInitIndexX, _gameInitIndexY];
							}
							cube = Instantiate(cubePrefab) as GameObject;
							cube.transform.parent = _gamePanel.transform;
							cube.transform.localScale = new Vector3 (1, 1, 1);
							//SetBirthPlace(cube, _gameInitIndexX, _gameInitIndexY);
							cube.transform.localPosition = GetCubePosition(_gameInitIndexX,_gameInitIndexY);
							cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
							cubeScript.SetGameCoordinate(_gameInitIndexX,_gameInitIndexY);
							cubeScript.StartMove();
							cubeScript.SetCubeType(cubeType);
							if(StageManager.GetInstance().BackgroundBlockChain(_gameInitIndexX, _gameInitIndexY))
							{
								cubeScript.EnableChain1();
							}else if(StageManager.GetInstance().BackgroundBlockDChain(_gameInitIndexX, _gameInitIndexY))
							{
								cubeScript.EnableChain2();
							}else if(StageManager.GetInstance().BackgroundBlockTChain(_gameInitIndexX, _gameInitIndexY))
							{
								cubeScript.EnableChain3();
							}else
							{
								cubeScript.DisableChain();
							}
							
							allCubes.Add(cube);
						}
						
						_gameInitIndexY++;
						if(_gameInitIndexY>=colum)
						{
							_gameInitIndexY = 0;
							_gameInitIndexX++;
						}
						if(_gameInitIndexX>=row)
						{
							ToState(GAME_STATE_READY);
							if(UserManager.GetInstance().Stage == 1 && !_trainedFlag)
							{
								_trainedFlag = true;
								MessageWindow.GetInstance().ShowWarning("֮ӻɻƽɫеĸӶɻƽɫͿԹأ");
							}
							if(UserManager.GetInstance().Stage == 2 && !_trainedFlag2)
							{
								_trainedFlag2 = true;
								MessageWindow.GetInstance().ShowWarning("ˮƽƶʯյĸڣúСɣ");
							}
							if(UserManager.GetInstance().Stage>5 && !UserManager.GetInstance().NotifiedForReview &&
								UserManager.GetInstance().Stage%3==0)
							{
								/*
								MessageWindow.GetInstance().ShowMessage("",
								                                        "5000ȥд5ۿллף",
									MessageWindow.MESSAGE_WINDOW_CUSTOMER_REVIEW);
									*/
							}
						}
					}
				}else
				{
					cubeType = GenerateRandomType();
					cube = Instantiate(cubePrefab) as GameObject;
					cube.transform.parent = _gamePanel.transform;
					cube.transform.localScale = new Vector3 (1, 1, 1);
					//SetBirthPlace(cube, _gameInitIndexX, _gameInitIndexY);
					cube.transform.localPosition = GetCubePosition(_gameInitIndexX,_gameInitIndexY);
					cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
					cubeScript.SetGameCoordinate(_gameInitIndexX,_gameInitIndexY);
					cubeScript.StartMove();
					cubeScript.SetCubeType(cubeType);
					
					allCubes.Add(cube);
					
					_gameInitIndexY++;
					if(_gameInitIndexY>=colum)
					{
						_gameInitIndexY = 0;
						_gameInitIndexX++;
					}
					if(_gameInitIndexX>=row)
					{
						ToState(GAME_STATE_READY);
					}
				}
			}
			_gameInitFrameIndex++;
		}
		
		if(birthQueue != null && birthQueue.Count > 0)
		{
			Debug.Log("birthQueueIndex="+birthQueueIndex);
			birthQueueIndex++;
			if((birthQueueIndex%GAME_INIT_FRMAE_GAP)==0)
			{
				birthOneCubeDo((int)birthQueue.Dequeue());
			}
		}
	}
	private static int[,] stage37 = {
		{1,3,2,3,5,5,4,1},
		{3,4,1,4,3,3,1,3},
		{2,2,4,3,4,1,3,4},
		{2,3,3,1,1,1,2,1},
		{3,1,1,4,2,4,2,1},
		{3,4,2,3,3,3,3,4},
		{1,4,1,4,4,5,4,3},
		{1,1,2,2,3,3,1,2}
	};
	
	public static GameEngine GetInstance()
	{
		return _instance;
	}
	
	public bool GameTimePlaying()
	{
		return _gamePlaying;
	}
	
	public bool GamePendingEnd()
	{
		return ( _pendingGameLose || _pendingGameVictory);
	}
	
	public void GameLosePending()
	{
		if(_pendingGameLose || _pendingGameVictory)
			return;
		
		_pendingFrameCount = 0;
		
		_pendingGameLose = true;
		GamePause();
	}
	
	public void GameVictoryPending()
	{
		if(_pendingGameLose || _pendingGameVictory)
			return;
		
		_pendingFrameCount = 0;
		
		_pendingGameVictory = true;
		_gamePendingV = true;
		
		System.DateTime now = System.DateTime.Now;
		double mSecs = now.Subtract(_gameStartTime).TotalMilliseconds;
		_timeBonusValue = timeLimit - (int)(mSecs/1000);
		
		//GamePause();
		
		//GameView.GetInstance().StageVictory();
	}
	
	public void GameReinit()
	{
		//destroy game
		GameCube script;
		int cubeCount = _gamePanel.transform.childCount;
		GameObject obj;
		for(int i=0; i<cubeCount; i++)
		{
			obj = _gamePanel.transform.GetChild(i).gameObject;
			
			if(obj == null)
				continue;
			
			script = obj.GetComponent(typeof(GameCube)) as GameCube;
			if(script != null)
				script.DestroyMeIm();
		}
		if(_cubeBackground != null)
		{
			Destroy(_cubeBackground);
		}
		DestroyGameBackground();
		
		//reinit game
		ToState(GAME_STATE_INIT);
		GenerateGameBackground();
		/**
		 * init cube data
		 * */
		InitCubeData();
		/**
		 * init cube focus
		 * */
		InitCubeFocus();
	}
	
	public void GameStart()
	{
		if(StageManager.GetInstance().CurrentStage<=1)
		{
			_cubeTypeCount = 5;
		}else if(StageManager.GetInstance().CurrentStage<=39)
		{
			_cubeTypeCount = 6;
		}else
		{
			_cubeTypeCount = 7;
		}
		
		timeLimit = 300;
		if(UserManager.GetInstance().Plus2Min)
			timeLimit = 420;
		if(UserManager.GetInstance().Plus5Min)
			timeLimit = 600;
		
		//StageCompletePanel.GetInstance().Show(false);
		
		ToState(GAME_STATE_INIT);
		
		if(!_gamePlaying)
			_gameStartTime = System.DateTime.Now;
		
		_gamePause = false;
		_pendingGameLose = false;
		_gamePlaying = true;
		_gamePendingV = false;
		
		StageManager.GetInstance().ReInitStageData(GameView.GetInstance().Stage);
		
		/**
		 * generate background image
		 * */
		GenerateGameBackground();
		
		SkillManager.GetInstance().InitSkill();
		
		/**
		 * init cube data
		 * */
		InitCubeData();
		
		/**
		 * init cube focus
		 * */
		InitCubeFocus();
		
		//MusicManager.GetInstance().PlayBGM(MusicManager.BGM_CLIP_ELECTRIC_ROMEO, 10);
	}
	
	private void InitCubeFocus()
	{
	}
	
	private void InitCubeData()
	{
		GameObject cube;
		GameCube cubeScript;
		int i,j;
		int cubeType;
		
		_data = new int[row,colum];
		_cubeData = new GameObject[row, colum];
		
		birthQueue = new Queue();
		fallingCubes = new ArrayList();
		allCubes = new ArrayList();
		
		_gameInitIndexX = 0;
		_gameInitIndexY = 0;
		_gameInitFrameIndex = 0;
		
		_pressedCubeX = -1;
		_pressedCubeY = -1;
	}
	
	private int ToState(int newState)
	{
		int oldState = _gameState;
		_gameState = newState;
		return oldState;
	}
	
	private bool InState(int state)
	{
		return _gameState == state;
	}
	
	private void SetBirthPlace(GameObject obj, int xCo, int yCo)
	{
		obj.transform.localPosition = new Vector3((yCo+1)*tileWidth-tileWidth/2, BIRTH_PLACE_OFFSET, 0);
	}
	
	private int GetCubeTypeEx1(int xCo, int yCo)
	{
		GameCube cubeScript;
		foreach(GameObject cube in allCubes)
		{
			cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			if(cubeScript.GetCOX()==xCo && cubeScript.GetCOY()==yCo)
			{
				return cubeScript.GetCubeType();
			}
		}
		return -1;
	}
	private int GenerateRandomTypeEx(int xCo, int yCo)
	{
		int t = Random.Range(1, _cubeTypeCount+1);
		
		//if(InState(GAME_STATE_INIT))
		{
			int t1 = -1, t2 = -1;
			int t3 = -1, t4 = -1;
			
			t1 = GetCubeTypeEx1(xCo-1, yCo);
			t2 = GetCubeTypeEx1(xCo-2, yCo);
			t3 = GetCubeTypeEx1(xCo, yCo-1);
			t4 = GetCubeTypeEx1(xCo, yCo-2);
			while((t==t1 && t==t2)||(t==t3 && t==t4))
			{
				t = Random.Range(1, _cubeTypeCount+1);
			}
		}

		return t;
	}
	private int GenerateRandomType()
	{
		return Random.Range(1, _cubeTypeCount+1);
	}
	
	public int GetMagicFactor()
	{
		switch(magicFactor)
		{
		case 2:
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_COMBO_2, 100, 3);
			break;
		case 3:
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_COMBO_3, 100, 3);
			break;
		case 4:
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_COMBO_4, 100, 3);
			break;
		case 5:
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_COMBO_5, 100, 3);
			break;
		case 6:
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_COMBO_6, 100, 3);
			break;
		case 7:
		case 8:
		case 9:
		case 10:
		case 11:
		case 12:
		case 13:
		case 14:
		case 15:
		case 16:
		case 17:
		case 18:
		case 19:
		case 20:
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_COMBO_7, 100, 3);
			break;
		}
		return magicFactor;
	}
	
	private bool CheckDisappear(int xCo, int yCo)
	{
		int num;
		int i;
		bool res = false;
		GameCube cubeScript;
		
		int magicUp = 0, magicDown = 0;
		
		if(_cubeData[xCo,yCo] == null)
			return res;
		
		//Debug.Log("CheckDisappear=================");
		
		/**
		 * check colum
		 * */
		num=1;
		//Debug.Log("["+xCo+","+yCo+"].type="+_data[xCo,yCo]);
		for(i=xCo-1; i>=0; i--)
		{
			if(IsCubeOfType(i, yCo, _data[xCo,yCo]))
			{
				//Debug.Log("["+i+","+yCo+"].type="+_data[xCo,yCo]);
				num++;
				magicDown++;
			}else
			{
				i=-1;
			}
		}
		for(i=xCo+1; i<row; i++)
		{
			if(IsCubeOfType(i, yCo, _data[xCo,yCo]))
			{
				//Debug.Log("["+i+","+yCo+"].type="+_data[xCo,yCo]);
				num++;
				magicUp++;
			}else
			{
				i=10;
			}
		}
		if(num>=3)
		{
			bool tempFlag = false;
			int delay = 0;
			//Debug.Log("Find one colum=========================num="+num);
			if(_switching)
			{
				//if(xCo==_switchingCube2X && yCo==_switchingCube2Y)
				{
					if(SkillManager.GetInstance().AddTypeEx(_data[xCo,yCo]))
					{
						//Debug.Log("Invoke skill "+_data[xCo,yCo]);
						tempFlag = true;
					}
				}
				//ComboManager.GetInstance().Action();
				//CharacterPanel.GetInstance().FocusCharacterByType(_data[xCo,yCo]);
				magicFactor = 1;
			}else
			{
				magicFactor++;
			}
			if(tempFlag)
			{
				InvokeSkillMagic(xCo, yCo, _data[xCo,yCo]);
			}
			//Debug.Log("["+xCo+","+yCo+"]");
			for(i=xCo+magicUp; i>xCo; i--)
			{
				if(IsCubeOfType(i, yCo, _data[xCo,yCo]))
				{
					if(tempFlag && _data[xCo,yCo]!=4)
					{
						InvokeSkillMagic(i, yCo, _data[xCo,yCo]);
					}
					DestroyCube(i,yCo, delay++);
					//Debug.Log("["+i+","+yCo+"]");
				}else
				{
					i=10;
				}
			}
			DestroyCube(xCo,yCo, delay++);
			for(i=xCo-1; i>=0; i--)
			{
				if(IsCubeOfType(i, yCo, _data[xCo,yCo]))
				{
					if(tempFlag && _data[xCo,yCo]!=4)
					{
						InvokeSkillMagic(i, yCo, _data[xCo,yCo]);
					}
					DestroyCube(i,yCo, delay++);
					//Debug.Log("["+i+","+yCo+"]");
				}else
				{
					i=-1;
				}
			}
			res = true;
		}
		/**
		 * check colum
		 * */
		num=1;
		for(i=yCo-1; i>=0; i--)
		{
			if(IsCubeOfType(xCo, i, _data[xCo,yCo]))
			{
				num++;
			}else
			{
				i=-1;
			}
		}
		for(i=yCo+1; i<colum; i++)
		{
			if(IsCubeOfType(xCo, i, _data[xCo,yCo]))
			{
				num++;
			}else
			{
				i=10;
			}
		}
		if(num>=3)
		{
			bool tempFlag = false;
			//Debug.Log("Find one row=========================num="+num);
			if(_switching)
			{
				//if(xCo==_switchingCube2X && yCo==_switchingCube2Y)
				{
					if(SkillManager.GetInstance().AddTypeEx(_data[xCo,yCo]))
					{
						tempFlag = true;
						//Debug.Log("Invoke skill "+_data[xCo,yCo]);
					}
				}
				//ComboManager.GetInstance().Action();
				//CharacterPanel.GetInstance().FocusCharacterByType(_data[xCo,yCo]);
				magicFactor=1;
			}else
			{
				magicFactor++;
			}
			DestroyCube(xCo,yCo);
			if(tempFlag)
			{
				InvokeSkillMagic(xCo, yCo, _data[xCo,yCo]);
			}
			//Debug.Log("["+xCo+","+yCo+"]");
			for(i=yCo-1; i>=0; i--)
			{
				if(IsCubeOfType(xCo, i, _data[xCo,yCo]))
				{
					if(tempFlag && _data[xCo,yCo]!=4)
					{
						InvokeSkillMagic(xCo, i, _data[xCo,yCo]);
					}
					DestroyCube(xCo,i);
					//Debug.Log("["+xCo+","+i+"]");
				}else
				{
					i=-1;
				}
			}
			for(i=yCo+1; i<row; i++)
			{
				if(IsCubeOfType(xCo, i, _data[xCo,yCo]))
				{
					if(tempFlag && _data[xCo,yCo]!=4)
					{
						InvokeSkillMagic(xCo, i, _data[xCo,yCo]);
					}
					DestroyCube(xCo,i);
					//Debug.Log("["+xCo+","+i+"]");
				}else
				{
					i=10;
				}
			}
			res = true;
		}
		if(res)
		{
			_moveCubeY = _moveCubeY = -1;
		}
		return res;
	}
	
	public void InvokeSkillMagic(int xCo, int yCo, int type)
	{
		GameObject obj;
		//Debug.Log("Invoke skill type="+type);
		return;
		switch(type)
		{
		case 1:
			//red
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_MAGIC_FIRE, 100, 3);			
			obj = Instantiate(magicSquarePrefab) as GameObject;
			obj.transform.parent = _gamePanel.transform;
			obj.transform.localScale = new Vector3(1,1,1);
			obj.transform.localPosition = this.GetCubePosition(xCo, yCo);
			for(int i=xCo-1; i<=xCo+1; i++)
			{
				for(int j=yCo-1; j<=yCo+1; j++)
				{
					DestroyCube(i, j);
				}
			}
			break;
		case 2:
			//orig
			break;
		case 3:
			//green
			obj = Instantiate(magicTimePrefab) as GameObject;
			obj.transform.parent = _gamePanel.transform;
			obj.transform.localScale = new Vector3(1,1,1);
			obj.transform.localPosition = this.GetCubePosition(xCo, yCo);
			break;
		case 4:
			//yellow
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_MAGIC_YELLOW, 100, 3);		
			int elimateType = Random.Range(1, _cubeTypeCount+1);
			GameCube cubeScript;
			for(int i=0; i<row; i++)
			{
				for(int j=0; j<colum; j++)
				{
					if(_cubeData[i,j] != null)
					{
						cubeScript = _cubeData[i,j].GetComponent(typeof(GameCube)) as GameCube;
						if(cubeScript.GetCubeType() == elimateType)
						{
							Debug.Log("yellow destroy cube type="+cubeScript.GetCubeType());
							DestroyCube(i,j);
						}
					}
				}
			}
			break;
		case 5:
			//blue
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_MAGIC_LIGHT, 100, 2);	
			obj = Instantiate(magicColumPrefab) as GameObject;
			obj.transform.parent = _gamePanel.transform;
			obj.transform.localScale = new Vector3(1,1,1);
			obj.transform.localPosition = this.GetCubePosition(0, yCo);
			for(int i=0; i<row; i++)
			{
				if(_cubeData[i, yCo] != null)
					DestroyCube(i, yCo);
			}
			break;
		case 6:
			//purple
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_MAGIC_LIGHT, 100, 2);	
			obj = Instantiate(magicRowPrefab) as GameObject;
			obj.transform.parent = _gamePanel.transform;
			obj.transform.localScale = new Vector3(1,1,1);
			obj.transform.localPosition = this.GetCubePosition(xCo, row/2);
			for(int i=0; i<colum; i++)
			{
				if(_cubeData[xCo,i] != null)
					DestroyCube(xCo, i);
			}
			break;
		}
	}
	
	public void AddTime(float second)
	{
		int timePast;
		System.DateTime now = System.DateTime.Now;
		double mSecs = now.Subtract(_gameStartTime).TotalMilliseconds;
		timePast = (int)(mSecs/1000);
		
		if(second > (timePast))
			second = timePast;
		
		_gameStartTime = _gameStartTime.AddSeconds(second);
		MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_MAGIC_CLOCK, 100, 3);
	}
	
	private bool CheckCanMove(int xCo, int yCo)
	{
		if(_pressedCubeX==xCo && _pressedCubeY==yCo)
		{
			return false;	
		}
		if(_pressedCubeX>=0 && _pressedCubeY>=0)
		{
			if((Mathf.Abs(xCo-_pressedCubeX)+Mathf.Abs(yCo-_pressedCubeY))==1)
			{
				return true;
			}
		}
		return false;
	}

	public bool NotifyClickCube(int xCo, int yCo, GameObject cube)
	{
		//Debug.Log("NotifyClickCube  enter  xCo="+xCo+",yCo="+yCo);
		
		_pressedCubeX = xCo;
		_pressedCubeY = yCo;
		
		if(_switching)
			return false;
		
		if(ISDestroying(xCo,yCo))
			return false;
		
		if(HaveMovingCubes())
			return false;
		
		return false;
	}
	
	private void ReSwitch()
	{
		GameCube script;
		
		_switching = true;
		_switchingPass = true;
		_switchingCount = 0;
		
		if(_cubeData[_switchingCube1X,_switchingCube1Y]==null)
			return;
		
		if(_cubeData[_switchingCube2X,_switchingCube2Y]==null)
			return;
		
		script = _cubeData[_switchingCube1X,_switchingCube1Y].GetComponent(typeof(GameCube)) as GameCube;
		script.SetGameCoordinate(_switchingCube2X,_switchingCube2Y);
		script.StartMove();
		_data[_switchingCube1X,_switchingCube1Y] = GAME_CUBE_SWITCHING;
		
		script = _cubeData[_switchingCube2X,_switchingCube2Y].GetComponent(typeof(GameCube)) as GameCube;
		script.SetGameCoordinate(_switchingCube1X,_switchingCube1Y);
		script.StartMove();
		_data[_switchingCube2X,_switchingCube2Y] = GAME_CUBE_SWITCHING;
		
		//GameCubeFocus.GetInstance().Show(false);
		
		ComboManager.GetInstance().Wrong();
	}
	
	private void Switch(int xCo, int yCo)
	{
		GameCube script;
		
		if(_cubeData[_pressedCubeX,_pressedCubeY]==null ||_cubeData[xCo,yCo]==null)
		{
			Debug.Log("Switch   one null");
			return;
		}
		
		_switching = true;
		_switchingPass = false;
		_switchingCube1X = _pressedCubeX;
		_switchingCube1Y = _pressedCubeY;
		_switchingCube2X = xCo;
		_switchingCube2Y = yCo;
		_switchingCount = 0;
		
		script = _cubeData[_pressedCubeX,_pressedCubeY].GetComponent(typeof(GameCube)) as GameCube;
		script.SetGameCoordinate(xCo,yCo);
		script.StartMove(1.2f);
		_data[_pressedCubeX,_pressedCubeY] = GAME_CUBE_SWITCHING;
		
		script = _cubeData[xCo,yCo].GetComponent(typeof(GameCube)) as GameCube;
		script.SetGameCoordinate(_pressedCubeX,_pressedCubeY);
		script.StartMove();
		_data[xCo,yCo] = GAME_CUBE_SWITCHING;
	}
	
	private void DestroyCube(int xCo, int yCo, int delayFrames = 0)
	{
		GameCube cubeScript;
		if(xCo<0 || xCo>=row)
			return;
		if(yCo<0 || yCo>=colum)
			return;
		if(_cubeData[xCo,yCo]==null)
			return;
		cubeScript = _cubeData[xCo,yCo].GetComponent(typeof(GameCube)) as GameCube;
		cubeScript.DestroyMe(delayFrames);
		//_data[xCo,yCo] = GAME_CUBE_DESTROYING;
	}
	
	private bool IsCubeOfType(int xCo, int yCo, int type)
	{
		if(xCo<0 || xCo>=row)
			return false;
		
		if(yCo<0 || yCo>=colum)
			return false;
		
		if(_cubeData[xCo,yCo] == null)
			return false;
		
		GameCube cubeScript = _cubeData[xCo,yCo].GetComponent(typeof(GameCube)) as GameCube;
		if(cubeScript.IsMoving())
			return false;
		
		return _data[xCo,yCo]==type;
	}
	
	private void DestroyGameBackground()
	{
		GameObject obj;
		int count;
		
		count = _backgroundPanel.transform.childCount;
		
		for(int i=0; i<count; i++)
		{
			Destroy(_backgroundPanel.transform.GetChild(i).gameObject);
		}
		
		if(_background1Panel != null)
		{
			_background1Panel.gameObject.active = false;
		}
	}
	
	public int GetCubeType(int x, int y)
	{
		return _data[x,y];
	}
	
	private void GenerateGameBackground()
	{
		int i, j;
		GameObject cubeBackground;
		GameObject cubeTest;
		//UILabel cubeTestLabel;
		UISprite sprite;
		
		if(_background1Panel != null)
		{
			_background1Panel.gameObject.active = true;
			
			/*
			UISprite character1, character2;
			character1 = _background1Panel.transform.FindChild("character1").GetComponent(typeof(UISprite)) as UISprite;
			character2 = _background1Panel.transform.FindChild("character2").GetComponent(typeof(UISprite)) as UISprite;
			int index = GameView.GetInstance().Stage/5+1;
			if(index<1)
				index=1;
			if(index>9)
				index=9;
			character1.spriteName = "character"+index.ToString();
			character2.spriteName = "character"+(index+1).ToString();
			*/
		}
		
		_backgroundBlock = new GameObject[row, colum];
		
		/*
		if(_cubeBackground == null)
		{
			Destroy(_cubeBackground);
		}
		
		_cubeBackground = Instantiate(cubeBackgroundPrefab) as GameObject;
		_cubeBackground.transform.parent = _backgroundPanel.transform;
		_cubeBackground.transform.localScale = new Vector3(colum*tileWidth, row*tileHeight, 1);
		_cubeBackground.transform.localPosition = new Vector3(
					0,
					0,
					1.1f
					);
		sprite = _cubeBackground.GetComponent(typeof(UISlicedSprite)) as UISlicedSprite;
		sprite.depth = -2;
		*/
		
		for(i=0; i<row; i++)
		{
			for(j=0; j<colum; j++)
			{
				/**
				 * test cube
				 * */
				if(testCube)
				{
					CubeTestScript testScript;
					cubeTest = Instantiate(cubeTestPrefab) as GameObject;
					cubeTest.transform.parent = _backgroundPanel.transform;
					cubeTest.transform.localScale = new Vector3(48, 48, 1);
					cubeTest.transform.localPosition = new Vector3(
						j*tileWidth,
						i*tileHeight,
						-4
						);
					testScript = cubeTest .GetComponent(typeof(CubeTestScript)) as CubeTestScript;
					testScript.SetCoordinates(i,j);
				}
				
				cubeBackground = Instantiate(cubeBackgroundPrefab) as GameObject;
				_backgroundBlock[i,j] = cubeBackground;
				cubeBackground.transform.parent = _backgroundPanel.transform;
				cubeBackground.transform.localScale = new Vector3(tileWidth, tileHeight, 1);
				cubeBackground.transform.localPosition = new Vector3(
					j*tileWidth,
					i*tileHeight,
					1
					);
				sprite = cubeBackground.GetComponent(typeof(UISprite)) as UISprite;
				if(StageManager.GetInstance().BackgroundBlockWall1(i,j))
				{
					//Debug.Log("Game background 777");
					sprite.spriteName = "cubeBackground7";
					//sprite.spriteName = "cubeBackground3";
					sprite.alpha = 1;
				}else if(StageManager.GetInstance().BackgroundBlockWall2(i,j))
				{
					//Debug.Log("Game background 888");
					sprite.spriteName = "cubeBackground8";
					//sprite.spriteName = "cubeBackground3";
					sprite.alpha = 1;
				}else if(StageManager.GetInstance().BackgroundBlockIce5(i,j))
				{
					sprite.spriteName = "cubeTerrainIce5";
					sprite.alpha = 1;
				}else if(StageManager.GetInstance().BackgroundBlockIce4(i,j))
				{
					sprite.spriteName = "cubeTerrainIce4";
					sprite.alpha = 1;
				}else if(StageManager.GetInstance().BackgroundBlockIce3(i,j))
				{
					sprite.spriteName = "cubeTerrainIce3";
					sprite.alpha = 1;
				}else if(StageManager.GetInstance().BackgroundBlockIce2(i,j))
				{
					sprite.spriteName = "cubeTerrainIce2";
					sprite.alpha = 1;
				}else if(StageManager.GetInstance().BackgroundBlockIce1(i,j))
				{
					sprite.spriteName = "cubeTerrainIce1";
					sprite.alpha = 1;
				}else if(StageManager.GetInstance().BackgroundBlockWall(i,j))
				{
					sprite.spriteName = "cubeBackground2";
					//sprite.spriteName = "cubeBackground3";
					sprite.alpha = 0;
				}else if(StageManager.GetInstance().BackgroundBlockGold(i,j))
				{
					if(SJQMode)
					sprite.spriteName = "cubeBackground1";
					else
						sprite.spriteName = "cubeBackground";
					//sprite.spriteName = "cubeBackground3";
					sprite.alpha = 1;
				}else if(StageManager.GetInstance().BackgroundBlockBlank(i,j))
				{
					//sprite.spriteName = "cubeBackground";
					sprite.spriteName = "cubeBackground";
					sprite.alpha = 1f;
				}else if(StageManager.GetInstance().BackgroundBlockGold_H(i,j))
				{
					if(SJQMode)
					sprite.spriteName = "cubeBackground2";
					else
						sprite.spriteName = "cubeBackground";
					sprite.alpha = 1f;
				}else
				{
					sprite.spriteName = "cubeBackground";
					sprite.alpha = 1f;
				}
				sprite.depth = -1;
			}
		}
	}
	
	public Vector3 GetCubePosition(int x, int y)
	{
		return new Vector3(y*tileWidth+tileWidth/2, x*tileHeight+tileHeight/2, -0.5f);
	}
	
	public float GetCubeMoveSpeed()
	{
		return _cubeMoveSpeed;
	}
	
	public float GetCubeFallSpeed()
	{
		return _cubeMoveSpeed;
	}
	
	private void FixColum(int col)
	{
		GameCube cubeScript;
		GameObject cube = null;
		bool[] flag = new bool[this.row];
		for(int i=0; i<7; i++)
		{
			flag[i] = false;
		}
		foreach(GameObject obj in allCubes)
		{
			cubeScript = obj.GetComponent(typeof(GameCube)) as GameCube;
			if(cubeScript.GetCOY()==col)
			{
				//Debug.Log("Cube["+cubeScript.GetCOX()+","+cubeScript.GetCOY()+"]");
				if(flag[cubeScript.GetCOX()])
				{
					cube = obj;
				}else
				{
					flag[cubeScript.GetCOX()] = true;
					_data[cubeScript.GetCOX(),col] = cubeScript.GetCubeType();
					_cubeData[cubeScript.GetCOX(),col] = obj;
				}
			}
		}
		if(cube != null)
		{
			int k;
			//Debug.Log("OOOOOOOOOOOOOOOOOOOOOOOOOOOfix a colum");
			for(k=0; k<7; k++)
			{
				if(flag[k] == false)
					break;
			}
			cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			cubeScript.SetGameCoordinate(k, col);
			cubeScript.StartMove();
		}
	}
	
	private bool CubeChain(int xCo, int yCo)
	{
		return (StageManager.GetInstance().BackgroundBlockChain(xCo,yCo) && _cubeData[xCo, yCo]);
	}
	
	private bool CubeDChain(int xCo, int yCo)
	{
		return (StageManager.GetInstance().BackgroundBlockDChain(xCo,yCo) && _cubeData[xCo, yCo]);
	}
	
	private bool CubeTChain(int xCo, int yCo)
	{
		return (StageManager.GetInstance().BackgroundBlockTChain(xCo,yCo) && _cubeData[xCo, yCo]);
	}
	
	public void CubeAtPosition(int xCo, int yCo, int type, GameObject cube)
	{
		if(_data[xCo, yCo] == GAME_CUBE_SWITCHING)
		{
			GameCube cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			if((cubeScript.GetCOX()==_switchingCube1X && cubeScript.GetCOY()==_switchingCube1Y) || 
				(cubeScript.GetCOX()==_switchingCube2X && cubeScript.GetCOY()==_switchingCube2Y))
			{
			}else
			{
				cubeScript.FallOneLevel();
				Debug.Log("KKKKKKKKKKKKKKKKKKKKKKKKKKKK at position fall one level");
				return;
			}
		}else
		{
			GameCube cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			if((cubeScript.GetCOX()==_switchingCube1X && cubeScript.GetCOY()==_switchingCube1Y) || 
				(cubeScript.GetCOX()==_switchingCube2X && cubeScript.GetCOY()==_switchingCube2Y))
			{
				//Debug.Log("IIIIIIIIIIIIIIIIIIIIIIIIII_data["+xCo+", "+yCo+"] = "+_data[xCo, yCo]);
				//if(_data[xCo, yCo]>0)
				{
					FixColum(yCo);
				}
			}
		}
		
		/*
		if(StageManager.GetInstance().BackgroundBlockWall(xCo,yCo)||
			CubeChain(xCo,yCo) ||
			CubeDChain(xCo,yCo))
		{
			GameCube cubeScript;
			
			int count = 0;
			for(int j=xCo; j>=0; j--)
			{
				if(StageManager.GetInstance().BackgroundBlockWall(j,yCo)||
					CubeChain(j,yCo) ||
					CubeDChain(j,yCo))
				{
					count++;
				}else{
					break;
				}
			}
			
			cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			//cubeScript.FallOneLevel();
			cubeScript.FallNLevel(count);
			//FallNLevel(xCo,yCo,count);
			Debug.Log("CubeAtPosition fall N level  N="+count+",["+xCo+","+yCo+"]");
			return ;
		}
		*/
		
		//Debug.Log("_data["+xCo+", "+yCo+"] = "+_data[xCo, yCo]);
		
		_data[xCo,yCo] = type;
		_cubeData[xCo,yCo] = cube;
		
		GameCube script = _cubeData[xCo,yCo].GetComponent(typeof(GameCube)) as GameCube;
		int prevXCo = script.GetPrevX();
		int prevYCo = script.GetPrevY();
		
		if( prevXCo>=0 && prevYCo>=0 && _cubeData[prevXCo,prevYCo] != null)
		{
			script = _cubeData[prevXCo, prevYCo].GetComponent(typeof(GameCube)) as GameCube;
			if(prevXCo != script.GetPrevX() || prevYCo != script.GetPrevY())
			{
				_cubeData[prevXCo,prevYCo] = null;
			}
		}
		
		bool res = CheckDisappear(xCo, yCo);
		
		if(!res 
			//&& xCo==_moveCubeX 
			//&& yCo==_moveCubeY 
			&& xCo>0 
			&& !StageManager.GetInstance().BackgroundBlockWall(xCo-1,yCo)
			&& !StageManager.GetInstance().BackgroundBlockChain(xCo-1,yCo)
			&& !StageManager.GetInstance().BackgroundBlockDChain(xCo-1,yCo)
			&& !StageManager.GetInstance().BackgroundBlockTChain(xCo-1,yCo)
			&& _data[xCo-1,yCo]==GAME_CUBE_NONE
			&& !HaveCubeWithTarget(xCo-1,yCo))
		{
			GameCube cubeScript;
			
			int count = 0;
			for(int j=xCo-1; j>=0; j--)
			{
				if(_data[j,yCo]==GAME_CUBE_NONE 
					&& !StageManager.GetInstance().BackgroundBlockWall(j,yCo)
					&& !StageManager.GetInstance().BackgroundBlockChain(j,yCo)
					&& !StageManager.GetInstance().BackgroundBlockDChain(j,yCo)
					&& !StageManager.GetInstance().BackgroundBlockTChain(j,yCo)
					&& !HaveCubeWithTarget(j,yCo)
					)
				{
					count++;
				}else{
					break;
				}
			}
			
			cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			cubeScript.FallNLevel(count);
			_data[xCo,yCo] = GAME_CUBE_NONE;
			FallNLevel(xCo,yCo,count);
			//Debug.Log("CubeAtPosition fall N level  1");
			
			_moveCubeY = _moveCubeY = -1;
			return ;
		}
		/*
		if(_moveCubeFlag && xCo==_moveCubeX && yCo==_moveCubeY)
		{
		}else
		{
			_moveCubeFlag = false;
		}
		*/
		
		if(_switching && ((xCo==_switchingCube1X && yCo==_switchingCube1Y) || ((xCo==_switchingCube2X && yCo==_switchingCube2Y))))
		{
			if(res)
			{
				_switchingPass = true;
				//GameCubeFocus.GetInstance().Show(false);
			}
			_switchingCount++;
			if(_switchingCount>=2)
			{
				if(_switchingPass)
				{
					_switching = false;
				}
				else
				{
					ReSwitch();
				}
			}
		}
		
		RemoveFalling (cube);
		
		if(!CheckIfCanSwitch())
		{
			Debug.Log("There is no swich you can do. Should restart this game!!!");
			//GameDestroy();
			
			//GameStart();
			GameReinit();
		}
	}
	
	private bool CheckIfCanWitchSingle(int xCo, int yCo, int type)
	{
		int num;
		int i;
		bool res = false;
		GameCube cubeScript;
		
		if(xCo<0 || xCo>=row)
			return res;
		if(yCo<0 || yCo>=colum)
			return res;
		
		/**
		 * check colum
		 * */
		num=1;
		//Debug.Log("["+xCo+","+yCo+"].type="+_data[xCo,yCo]);
		for(i=xCo-1; i>=0; i--)
		{
			if(IsCubeOfType(i, yCo, type))
			{
				//Debug.Log("["+i+","+yCo+"].type="+_data[xCo,yCo]);
				num++;
			}else
			{
				i=-1;
			}
		}
		for(i=xCo+1; i<row; i++)
		{
			if(IsCubeOfType(i, yCo, type))
			{
				//Debug.Log("["+i+","+yCo+"].type="+_data[xCo,yCo]);
				num++;
			}else
			{
				i=10;
			}
		}
		if(num>=3)
		{
			res = true;
		}
		/**
		 * check colum
		 * */
		num=1;
		for(i=yCo-1; i>=0; i--)
		{
			if(IsCubeOfType(xCo, i, type))
			{
				num++;
			}else
			{
				i=-1;
			}
		}
		for(i=yCo+1; i<colum; i++)
		{
			if(IsCubeOfType(xCo, i, type))
			{
				num++;
			}else
			{
				i=10;
			}
		}
		if(num>=3)
		{
			res = true;
		}
		return res;
	}
	bool HaveCubeWithTarget(int xCo, int yCo)
	{
		GameCube cubeScript;
		foreach(GameObject cube in allCubes)
		{
			cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			
			if(cubeScript.GetCOX()==xCo && cubeScript.GetCOY()==yCo)
			{
				return true;
			}
		}
		return false;
	}
	private bool HaveMovingCubes()
	{
		GameCube cubeScript;
		foreach (GameObject cube in allCubes)
		{
			cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			if(cubeScript.IsMoving() || cubeScript.IsDestroying() || cubeScript.IsFalling())
				return true;
		}
		return false;
	}
	private void StoreHint(int x, int y, int direction)
	{
		hintX = x;
		hintY = y;
		hintDirection = direction;
	}
	/**
	 * check if there is a valid switch user can do
	 * if there is no.
	 * then reinit game.
	 * */
	private bool CheckIfCanSwitch()
	{
		int i, j, temp;
		//Debug.Log("CheckIfCanSwitch  enter");
		
		if(_switching)
		{
			//Debug.Log("CheckIfCanSwitch  _switching");
			return true;
		}
		
		/*
		if(InState(GAME_STATE_INIT))
			return true;
			*/
		
		for(i=0; i<row; i++)
		{
			for(j=0; j<colum; j++)
			{
				if(HaveMovingCubes())
				{
					//Debug.Log("CheckIfCanSwitch  have moving cubes");
					return true;
				}
				
				if(StageManager.GetInstance().BackgroundBlockChain(i,j) ||
					StageManager.GetInstance().BackgroundBlockDChain(i,j) ||
					StageManager.GetInstance().BackgroundBlockTChain(i,j) ||
					StageManager.GetInstance().BackgroundBlockWall(i,j))
					continue;
				
				if(!InState(GAME_STATE_INIT) && _data[i,j]==GAME_CUBE_NONE)
					continue;
				
				temp = _data[i,j];
				
				if(i>=1)
					_data[i,j] = _data[i-1,j];
				if(!StageManager.GetInstance().BackgroundBlockChain(i-1,j) &&
					!StageManager.GetInstance().BackgroundBlockDChain(i-1,j) &&
					!StageManager.GetInstance().BackgroundBlockTChain(i-1,j) &&
					!StageManager.GetInstance().BackgroundBlockWall(i-1,j) &&
					CheckIfCanWitchSingle(i-1,j,temp))
				{
					//Debug.Log("CheckIfCanSwitch 1  i="+i+",j="+j+" - 1");
					_data[i,j] = temp;
					StoreHint(i, j, HINT_DRECTION_DOWN);
					return true;
				}
				if(i>=1 && _data[i-1,j]==GAME_CUBE_NONE && 
					!StageManager.GetInstance().BackgroundBlockWall(i-1,j))
				{
					//Debug.Log("CheckIfCanSwitch 2  i="+i+",j="+j+" - 1");
					_data[i,j] = temp;
					StoreHint(i, j, HINT_DRECTION_DOWN);
					return true;
				}
					
				_data[i,j] = temp;
				if(i<row-1)
					_data[i,j] = _data[i+1,j];
				if(!StageManager.GetInstance().BackgroundBlockChain(i+1,j) &&
					!StageManager.GetInstance().BackgroundBlockDChain(i+1,j) &&
					!StageManager.GetInstance().BackgroundBlockTChain(i+1,j) &&
					!StageManager.GetInstance().BackgroundBlockWall(i+1,j) &&
					CheckIfCanWitchSingle(i+1,j,temp))
				{
					//Debug.Log("CheckIfCanSwitch 3  i="+i+",j="+j+" - 2");
					_data[i,j] = temp;
					StoreHint(i, j, HINT_DRECTION_UP);
					return true;
				}
				/*
				if(i<row-1 && _data[i+1,j]==GAME_CUBE_NONE && 
					!StageManager.GetInstance().BackgroundBlockWall(i+1,j))
				{
					Debug.Log("CheckIfCanSwitch 4  i="+i+",j="+j+" - 2");
					_data[i,j] = temp;
					return true;
				}
				*/
				
				_data[i,j] = temp;
				if(j>=1)
					_data[i,j] = _data[i,j-1];
				if(!StageManager.GetInstance().BackgroundBlockChain(i,j-1) &&
					!StageManager.GetInstance().BackgroundBlockDChain(i,j-1) &&
					!StageManager.GetInstance().BackgroundBlockTChain(i,j-1) &&
					!StageManager.GetInstance().BackgroundBlockWall(i,j-1) &&
					CheckIfCanWitchSingle(i,j-1,temp))
				{
					//Debug.Log("CheckIfCanSwitch 5  i="+i+",j="+j+" - 3");
					_data[i,j] = temp;
					StoreHint(i, j, HINT_DRECTION_LEFT);
					return true;
				}
				if(j>=1 && _data[i,j-1]==GAME_CUBE_NONE && 
					!StageManager.GetInstance().BackgroundBlockWall(i,j-1))
				{
					//Debug.Log("CheckIfCanSwitch 6  i="+i+",j="+j+" - 3");
					_data[i,j] = temp;
					StoreHint(i, j, HINT_DRECTION_LEFT);
					return true;
				}
				
				_data[i,j] = temp;
				if(j<colum-1)
					_data[i,j] = _data[i,j+1];
				if(!StageManager.GetInstance().BackgroundBlockChain(i,j+1) &&
					!StageManager.GetInstance().BackgroundBlockDChain(i,j+1) &&
					!StageManager.GetInstance().BackgroundBlockTChain(i,j+1) &&
					!StageManager.GetInstance().BackgroundBlockWall(i,j+1) &&
					CheckIfCanWitchSingle(i,j+1,temp))
				{
					//Debug.Log("CheckIfCanSwitch 7  i="+i+",j="+j+" - 4");
					_data[i,j] = temp;
					StoreHint(i, j, HINT_DRECTION_RIGHT);
					return true;
				}
				if(j<colum-1 && _data[i,j+1]==GAME_CUBE_NONE && 
					!StageManager.GetInstance().BackgroundBlockWall(i,j+1))
				{
					//Debug.Log("CheckIfCanSwitch 8  i="+i+",j="+j+" - 4");
					_data[i,j] = temp;
					StoreHint(i, j, HINT_DRECTION_RIGHT);
					return true;
				}
				
				_data[i,j] = temp;
			}
		}
		//Debug.Log("CheckIfCanSwitch  No switch you can do");
		return false;
	}
	
	private void DebugColum(int c)
	{
		for(int i=0; i<row; i++)
		{
			Debug.Log("_data["+i+","+c+"]="+_data[i,c]);
			Debug.Log("_cubeData["+i+","+c+"]="+_cubeData[i,c]);
		}
	}
	
	public void NotifyCubeNChained(int xCo, int yCo, int type, GameObject cube)
	{
		if(HaveMovingCubes())
		{
			if(xCo>0 && !StageManager.GetInstance().BackgroundBlockWall(xCo-1,yCo)
				&& !StageManager.GetInstance().BackgroundBlockChain(xCo-1,yCo)
				&& !StageManager.GetInstance().BackgroundBlockDChain(xCo-1,yCo)
				&& !StageManager.GetInstance().BackgroundBlockTChain(xCo-1,yCo)
				&& _data[xCo-1,yCo]==GAME_CUBE_NONE
				&& !HaveCubeWithTarget(xCo,yCo))
			{
				GameCube cubeScript;
				
				int count = 0;
				for(int j=xCo-1; j>=0; j--)
				{
					if(_data[j,yCo]==GAME_CUBE_NONE
						&& !StageManager.GetInstance().BackgroundBlockWall(j,yCo)
						&& !StageManager.GetInstance().BackgroundBlockChain(j,yCo)
						&& !StageManager.GetInstance().BackgroundBlockDChain(j,yCo)
						&& !StageManager.GetInstance().BackgroundBlockTChain(j,yCo)
						&& !HaveCubeWithTarget(j,yCo))
					{
						count++;
					}else{
						break;
					}
				}
				
				cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
				cubeScript.FallNLevel(count);
				//Debug.Log("NotifyCubeNChained  fall n level  2   N="+count+",["+xCo+","+yCo+"]");
				_data[xCo,yCo] = GAME_CUBE_NONE;
				FallNLevel(xCo,yCo,count);
				return ;
			}else
			{
				//Debug.Log("NotifyCubeNChained 1 CubeAtPosition  ,["+xCo+","+yCo+"]");
				CubeAtPosition(xCo,yCo,type,cube);
			}
		}else
		{
			_moveCubeX = xCo;
			_moveCubeY = yCo;
			CubeAtPosition(xCo,yCo,type,cube);
		}
		return;
		
		if(CheckDisappear(xCo,yCo))
		{
			return;
		}
		if(xCo>0 && !StageManager.GetInstance().BackgroundBlockWall(xCo-1,yCo)
			&& !StageManager.GetInstance().BackgroundBlockChain(xCo-1,yCo)
			&& !StageManager.GetInstance().BackgroundBlockDChain(xCo-1,yCo)
			&& !StageManager.GetInstance().BackgroundBlockTChain(xCo-1,yCo)
			&& _data[xCo-1,yCo]==GAME_CUBE_NONE)
		{
			GameCube cubeScript;
			
			int count = 0;
			for(int j=xCo-1; j>=0; j--)
			{
				if(_data[j,yCo]==GAME_CUBE_NONE
					&& !StageManager.GetInstance().BackgroundBlockWall(j,yCo)
					&& !StageManager.GetInstance().BackgroundBlockChain(j,yCo)
					&& !StageManager.GetInstance().BackgroundBlockDChain(j,yCo)
					&& !StageManager.GetInstance().BackgroundBlockTChain(j,yCo))
				{
					count++;
				}else{
					break;
				}
			}
			
			cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			cubeScript.FallNLevel(count);
			Debug.Log("NotifyCubeNChained  fall n level  2   N="+count+",["+xCo+","+yCo+"]");
			_data[xCo,yCo] = GAME_CUBE_NONE;
			FallNLevel(xCo,yCo,count);
			return ;
		}else
		{
			//CheckDisappear(xCo,yCo);
			if(!CheckIfCanSwitch())
			{
				Debug.Log("There is no swich you can do. Should restart this game!!!");
				GameReinit();
			}
		}
	}
	
	public void NotifyCubeDragged(int xCo, int yCo, int direction, GameObject cube)
	{
		int tx = _pressedCubeX, ty = _pressedCubeY;
		
		//Debug.Log("NotifyCubeDragged ()()()()()()()()()()()()()()()()()()()()()()");
		
		if(_pendingGameLose || _pendingGameVictory || _gamePendingV)
			return;
		
		if(HaveMovingCubes())
			return;
		
		//Debug.Log("NotifyCubeDragged   direction="+direction);
		if(ISDestroying(xCo,yCo))
		{
			Debug.Log("cube destroying");
			return;
		}
		
		if(StageManager.GetInstance().BackgroundBlockChain(xCo,yCo))
		{
			return;
		}
		
		if(StageManager.GetInstance().BackgroundBlockDChain(xCo,yCo))
		{
			return;
		}
			
		if(StageManager.GetInstance().BackgroundBlockTChain(xCo,yCo))
		{
			return;
		}
			
		if(_switching)
		{
			Debug.Log("cube switching");
			return;
		}
		switch(direction)
		{
		case GameCube.TOUCH_DIRECTION_NONE:
			return;
			break;
		case GameCube.TOUCH_DIRECTION_LEFT:
			ty = _pressedCubeY-1;
			break;
		case GameCube.TOUCH_DIRECTION_RIGHT:
			ty = _pressedCubeY+1;
			break;
		case GameCube.TOUCH_DIRECTION_DOWN:
			tx = _pressedCubeX-1;
			break;
		case GameCube.TOUCH_DIRECTION_UP:
			tx = _pressedCubeX+1;
			break;
		}
		if(tx<0 || tx>=row)
		{
			Debug.Log("tx<0 || tx>=row");
			return;
		}
		if(ty<0 || ty>=colum)
		{
			Debug.Log("ty<0 || ty>=colum");
			return;
		}
		if(_data[tx,ty]==0 && !StageManager.GetInstance().BackgroundBlockWall(tx,ty))
		{
			GameCube cubeScript1;
			//Debug.Log("NotifycubeDragged   move to a blank ");
			cubeScript1 = cube.GetComponent(typeof(GameCube)) as GameCube;
			cubeScript1.SetGameCoordinate(tx,ty);
			cubeScript1.StartMove();
			_data[xCo,yCo] = GAME_CUBE_NONE;
			_cubeData[xCo,yCo] = null;
			
			//notify cube destroyed
			if(direction != GameCube.TOUCH_DIRECTION_UP)
			{
				if(_data[xCo,yCo]==GAME_CUBE_DESTROYING)
					_data[xCo,yCo] = GAME_CUBE_NONE;
				FallOneLevel(xCo, yCo);
			}
			
			_moveCubeFlag = true;
			_moveCubeX = tx;
			_moveCubeY = ty;
			
			return;
		}else
		{
			//Debug.Log("NotifyCubeDragged target ="+_data[tx,ty]);
		}
		
		
		if(_data[tx,ty]<0)
		{
			Debug.Log("_data[tx,ty]==null");
			return;
		}
		if(_cubeData[tx,ty]==null)
		{
			Debug.Log("_cubeData[tx,ty]==null");
			return;
		}
		
		if(StageManager.GetInstance().BackgroundBlockChain(tx,ty))
		{
			return;
		}
		
		if(StageManager.GetInstance().BackgroundBlockDChain(tx,ty))
		{
			return;
		}
		
		if(StageManager.GetInstance().BackgroundBlockTChain(tx,ty))
		{
			return;
		}
		
		GameCube cubeScript;
		cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
		_data[xCo,yCo] = cubeScript.GetCubeType();
		_cubeData[xCo,yCo] = cube;
		
		//GameCube script;
		//script = _cubeData[tx,ty].GetComponent(typeof(GameCube)) as GameCube;
		//Debug.Log("tx="+tx+",ty="+ty);
		if(ISDestroying(tx,ty))
		{
			Debug.Log("target destroying");
			return;
		}
		
		_draggedCubeX = xCo;
		_draggedCubeY = yCo;
		
		Switch(tx,ty);
	}
	
	public void NotifyWallDestroyed(int xCo, int yCo, int lastX, int lastY)
	{
		_data[xCo,yCo] = GAME_CUBE_NONE;
	
		{
			StageManager.GetInstance().SetBackgroundBlank(lastX, lastY);
			if(_backgroundBlock[lastX,lastY] != null)
			{
				//Debug.Log("NotifyCubeDestroyed   gold background  ["+xCo+","+yCo+"]");
				UISprite backgroundSprite = _backgroundBlock[lastX,lastY].GetComponent(typeof(UISprite)) as UISprite;
				backgroundSprite.spriteName = "cubeBackground";
				backgroundSprite.alpha = 1;
				backgroundSprite.MarkAsChanged();
			}else
			{
				//Debug.Log("NotifyCubeDestroyed   _backgroundBlock[xCo,yCo] == null");
			}
		}
		
		if(xCo>0 && !StageManager.GetInstance().BackgroundBlockWall(xCo-1,yCo)
			&& !StageManager.GetInstance().BackgroundBlockChain(xCo-1,yCo)
			&& !StageManager.GetInstance().BackgroundBlockDChain(xCo-1,yCo)
			&& !StageManager.GetInstance().BackgroundBlockTChain(xCo-1,yCo)
			&& _data[xCo-1,yCo]==GAME_CUBE_NONE
			&& !HaveCubeWithTarget(xCo,yCo))
		{
			int count = 0;
			for(int j=xCo-1; j>=0; j--)
			{
				if(_data[j,yCo]==GAME_CUBE_NONE
					&& !StageManager.GetInstance().BackgroundBlockWall(j,yCo)
					&& !StageManager.GetInstance().BackgroundBlockChain(j,yCo)
					&& !StageManager.GetInstance().BackgroundBlockDChain(j,yCo)
					&& !StageManager.GetInstance().BackgroundBlockTChain(j,yCo)
					&& !HaveCubeWithTarget(j,yCo))
				{
					count++;
				}else{
					break;
				}
			}
			
			Debug.Log("NotifyCubeNChained  fall n level  2   N="+count+",["+xCo+","+yCo+"]");
			_data[xCo,yCo] = GAME_CUBE_NONE;
			FallNLevel(xCo,yCo,count+1);
		}else
		{
			FallOneLevel(xCo, yCo);
		}
		
	}
	
	public void NotifyCubeDestroyed(int xCo, int yCo, GameObject cube, int lastX, int lastY, bool checkWall=true)
	{
		//if(_data[xCo,yCo]==GAME_CUBE_DESTROYING)
			_data[xCo,yCo] = GAME_CUBE_NONE;
		//Debug.Log("NotifyCubeDestroyed  enter");
		if(StageManager.GetInstance().BackgroundBlockBLANK_W(lastX, lastY))
		{
			StageManager.GetInstance().SetBackgroundGoldH(lastX, lastY);
			if(_backgroundBlock[lastX,lastY] != null)
			{
				//Debug.Log("NotifyCubeDestroyed   gold background  ["+xCo+","+yCo+"]");
				UISprite backgroundSprite = _backgroundBlock[lastX,lastY].GetComponent(typeof(UISprite)) as UISprite;
				if(SJQMode)
				backgroundSprite.spriteName = "cubeBackground2";
				else
					backgroundSprite.spriteName = "cubeBackground";
				backgroundSprite.alpha = 1;
				backgroundSprite.MarkAsChanged();
			}else
			{
				//Debug.Log("NotifyCubeDestroyed   _backgroundBlock[xCo,yCo] == null");
			}
		}else
		{
			//Debug.Log("NotifyCubeDestroyed  SetBackgroundGold");
		StageManager.GetInstance().SetBackgroundGold(lastX, lastY);
		if(_backgroundBlock[lastX,lastY] != null)
		{
			//Debug.Log("NotifyCubeDestroyed   gold background  ["+xCo+","+yCo+"]");
			UISprite backgroundSprite = _backgroundBlock[lastX,lastY].GetComponent(typeof(UISprite)) as UISprite;
				if(SJQMode)
			backgroundSprite.spriteName = "cubeBackground1";
				else
					backgroundSprite.spriteName = "cubeBackground";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
		}else
		{
			//Debug.Log("NotifyCubeDestroyed   _backgroundBlock[xCo,yCo] == null");
		}
		}
		
		
		allCubes.Remove(cube);
		
		//Debug.Log("Notify cube destroyed   ["+lastX+","+lastY+"]  call fall one level");
		FallOneLevel(xCo, yCo);
		
		if(checkWall)
		{
			CheckDestroyTypeWall(xCo-1, yCo);
			CheckDestroyTypeWall(xCo+1, yCo);
			CheckDestroyTypeWall(xCo, yCo-1);
			CheckDestroyTypeWall(xCo, yCo+1);
		}
		
		//birthOneCubeDo(yCo);
	}
	
	public void DestroyBackgroundChui(int xCo, int yCo)
	{
	}
	public void DestroyWallByChui(int xCo, int yCo)
	{
		if(xCo<0 || xCo>=row)
			return;
		if(yCo<0 || yCo>=colum)
			return;
		
		if(StageManager.GetInstance().BackgroundBlockBlank(xCo, yCo))
		{
			StageManager.GetInstance().SetBackgroundGold(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeBackground1";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_FALL_GROUND, 100, 3);
			
			return;
		}
		
		if(StageManager.GetInstance().BackgroundBlockIce5(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundIce4(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeTerrainIce4";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockIce4(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundIce3(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeTerrainIce3";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockIce3(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundIce2(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeTerrainIce2";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockIce2(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundIce1(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeTerrainIce1";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockIce1(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundBlank(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeBackground";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			NotifyWallDestroyed(xCo, yCo, xCo, yCo);
			
			return;
		}

		if(StageManager.GetInstance().BackgroundBlockWall2(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundWall1(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeBackground7";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ROCK, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyWallEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			/*
			obj.transform.localPosition = new Vector3(
				obj.transform.localPosition.x + tileWidth/2,
				obj.transform.localPosition.y + tileHeight/2,
				obj.transform.localPosition.z
				);
				*/
			
			//NotifyWallDestroyed(xCo, yCo, xCo, yCo);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockWall1(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundBlank(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeBackground";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ROCK, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyWallEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			/*
			obj.transform.localPosition = new Vector3(
				obj.transform.localPosition.x + tileWidth/2,
				obj.transform.localPosition.y + tileHeight/2,
				obj.transform.localPosition.z
				);
				*/
			
			NotifyWallDestroyed(xCo, yCo, xCo, yCo);
			
			return;
		}
	}
	
	private void CheckDestroyTypeWall(int xCo, int yCo)
	{
		if(xCo<0 || xCo>=row)
			return;
		if(yCo<0 || yCo>=colum)
			return;
		
		if(StageManager.GetInstance().BackgroundBlockIce5(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundIce4(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeTerrainIce4";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockIce4(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundIce3(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeTerrainIce3";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockIce3(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundIce2(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeTerrainIce2";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockIce2(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundIce1(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeTerrainIce1";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockIce1(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundBlank(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeBackground";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ICE, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyIceEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockWall2(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundWall1(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeBackground7";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ROCK, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyWallEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			
			return;
		}
		if(StageManager.GetInstance().BackgroundBlockWall1(xCo,yCo))
		{
			StageManager.GetInstance().SetBackgroundBlank(xCo,yCo);
			
			UISprite backgroundSprite = _backgroundBlock[xCo, yCo].GetComponent(typeof(UISprite)) as UISprite;
			backgroundSprite.spriteName = "cubeBackground";
			backgroundSprite.alpha = 1;
			backgroundSprite.MarkAsChanged();
			
			MusicManager.GetInstance().PlaySE(MusicManager.SE_CLIP_ROCK, 100, 2);
			
			GameObject obj;
			obj = Instantiate(DestroyWallEffectPrefab) as GameObject;
			obj.transform.position = new Vector3(
				backgroundSprite.transform.position.x,
				backgroundSprite.transform.position.y,
				backgroundSprite.transform.position.z-0.1f
				);
			/*
			obj.transform.localPosition = new Vector3(
				obj.transform.localPosition.x + tileWidth/2,
				obj.transform.localPosition.y + tileHeight/2,
				obj.transform.localPosition.z
				);
				*/
			
			NotifyWallDestroyed(xCo, yCo, xCo, yCo);
			
			return;
		}
	}
	
	private void UpdateCubeObject(int xCo, int yCo)
	{
		if(_cubeData[xCo,yCo] != null)
		{
			GameCube script;
			script = _cubeData[xCo,yCo].GetComponent(typeof(GameCube)) as GameCube;
			if(script.IsFalling())
				return;
			
			if(xCo!=script.GetCOX() || yCo!=script.GetCOY())
			{
				_cubeData[xCo,yCo] = null;
			}
		}
	}
	
	public void NotifyFalling(int xCo, int yCo, GameObject cube)
	{
		//Debug.Log("NotifyFalling enter");
		if(xCo < 0 || xCo >= row)
			return;
		
		if(yCo <0 || yCo >= colum)
			return;
		
		_cubeData[xCo, yCo] = null;
	}
	
	public void NotifyDestroying(int xCo, int yCo, GameObject cube)
	{
		_data[xCo, yCo] = GAME_CUBE_DESTROYING;
		//_cubeData[xCo, yCo] = null;
	}
	
	private void birthOneCube(int colum)
	{
		birthQueue.Enqueue(colum);
	}
	
	private void birthOneCubeDo(int colum)
	{
		GameObject cube;
		GameCube cubeScript;
		int cubeType;
		int count = 0;
		//Debug.Log("Birth a cube do.");
		
			for(int j=row-1; j>=0; j--)
			{
				if(StageManager.GetInstance().BackgroundBlockWall(j,colum)
				|| StageManager.GetInstance().BackgroundBlockChain(j,colum)
				|| StageManager.GetInstance().BackgroundBlockDChain(j,colum)
				|| StageManager.GetInstance().BackgroundBlockTChain(j,colum)
				)
				{
					count++;
				}else{
					break;
				}
			}
		
		/**
		 * birth a cube
		 * */
		if(allCubes.Count < (row*colum+StageManager.GetInstance().WallBlockNum))
		{
			cubeType = GenerateRandomType();
			cube = Instantiate(cubePrefab) as GameObject;
			cube.transform.parent = _gamePanel.transform;
			cube.transform.localScale = new Vector3 (1, 1, 1);
			//SetBirthPlace(cube, row-1, colum);
			cube.transform.localPosition = new Vector3(
				(colum+1)*tileWidth-tileWidth/2, 
				BIRTH_PLACE_OFFSET+fallingCubes.Count*tileWidth, 
				0
				);
			cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
			Debug.Log("Birth one cube fall N levels N="+count);
			cubeScript.SetGameCoordinate(row-1-count,colum);
			cubeScript.StartMove();
			cubeScript.SetCubeType(cubeType);
			
			AddFalling(cube);
			
			allCubes.Add(cube);
		}
	}
	
	private void AddFalling(GameObject obj)
	{
		//Debug.Log("++++++++++++++Add a falling object");
		fallingCubes.Add(obj);
	}
	
	private void RemoveFalling(GameObject obj)
	{
		if(fallingCubes.Contains(obj))
		{
			//Debug.Log("++++++++++++++Remove a falling object");
			fallingCubes.Remove(obj);
		}
	}
	
	public void FallNLevel(int xCo, int yCo, int N)
	{
		bool wallFlag = false;
		bool noBirth = false;
		int wallPos = 0;
		GameCube script;
		
		//Debug.Log("FallOneLevel  enter  [x="+xCo+",y="+yCo+"]");
		
		for(int i=xCo; i<row; i++)
		{
			if(wallFlag)
			{
				if(StageManager.GetInstance().BackgroundBlockWall(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockChain(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockDChain(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockTChain(i,yCo) )
				{
					continue;
				}else{
					noBirth = true;
				}
			}else
			{
				if(StageManager.GetInstance().BackgroundBlockWall(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockChain(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockDChain(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockTChain(i,yCo))
				{
					wallFlag = true;
					wallPos = i;
					//Debug.Log("FallOneLevel   find wall  [i="+i+",y="+yCo+"]");
					continue;
				}
			}
		}
		foreach(GameObject obj in allCubes)
		{
			script = obj.GetComponent(typeof(GameCube)) as GameCube;
			if(script.GetCOY() == yCo && script.GetCOX()>xCo)
			{
				if(wallFlag)
				{
					if(script.GetCOX()<=wallPos 
						&& !StageManager.GetInstance().BackgroundBlockChain(script.GetCOX(),script.GetCOY())
						&& !StageManager.GetInstance().BackgroundBlockDChain(script.GetCOX(),script.GetCOY())
						&& !StageManager.GetInstance().BackgroundBlockTChain(script.GetCOX(),script.GetCOY()))
					{
						_data[script.GetCOX(), script.GetCOY()] = 0;
						script.FallNLevel(N);
						//Debug.Log("FallNLevel  [x="+xCo+",y="+yCo+"]    DO");
					}
				}else
				{
					_data[script.GetCOX(), script.GetCOY()] = 0;
					script.FallNLevel(N);
					//Debug.Log("FallNLevel  [x="+xCo+",y="+yCo+"]    DO");
				}
					
			}
		}
		if(!noBirth)
		{
			GameObject cube;
			GameCube cubeScript;
			int cubeType;
			int count = 0;
			
			for(int j=row-1; j>=0; j--)
			{
				if(StageManager.GetInstance().BackgroundBlockWall(j,yCo)
					|| StageManager.GetInstance().BackgroundBlockChain(j,yCo)
					|| StageManager.GetInstance().BackgroundBlockDChain(j,yCo)
					|| StageManager.GetInstance().BackgroundBlockTChain(j,yCo))
				{
					count++;
				}else{
					break;
				}
			}
			
			/**
			 * birth a cube
			 * */
			//Debug.Log("birth a cube   wallPos="+wallPos);
			//Debug.Log("birth a cube   yCo="+yCo);
			for(int n=0; n<N; n++)
			{
				if(allCubes.Count < (row*colum+StageManager.GetInstance().WallBlockNum))
				{
					cubeType = GenerateRandomType();
					cube = Instantiate(cubePrefab) as GameObject;
					cube.transform.parent = _gamePanel.transform;
					cube.transform.localScale = new Vector3 (1, 1, 1);
					//SetBirthPlace(cube, row-1, colum);
					cube.transform.localPosition = new Vector3(
						(yCo+1)*tileWidth-tileWidth/2, 
						BIRTH_PLACE_OFFSET+fallingCubes.Count*tileWidth, 
						0
						);
					cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
					//Debug.Log("FallOneLevel birth one cube fall N levels   N="+count);
					cubeScript.SetGameCoordinate(row-1-count-(N-n-1),yCo);
					cubeScript.StartMove();
					cubeScript.SetCubeType(cubeType);
					//Debug.Log("NotifyCubeNChained  fall n Level birth a cube,["+(row-1-count-(N-n-1))+","+yCo+"]");
					
					AddFalling(cube);
					
					allCubes.Add(cube);
				}
			}
		}
	}
	
	public void FallOneLevel(int xCo, int yCo)
	{
		bool wallFlag = false;
		bool noBirth = false;
		int wallPos = 0;
		GameCube script;
		
		//Debug.Log("FallOneLevel  enter  [x="+xCo+",y="+yCo+"]");
		
		for(int i=xCo; i<row; i++)
		{
			if(wallFlag)
			{
				if(StageManager.GetInstance().BackgroundBlockWall(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockChain(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockDChain(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockTChain(i,yCo))
				{
					continue;
				}else
				{
					noBirth = true;
				}
				/*
				if(_data[i,yCo]>=1 && _data[i,yCo]<=_cubeTypeCount && _cubeData[i,yCo]!=null)
				{
					noBirth = true;
				}
				*/
			}else
			{
				if(StageManager.GetInstance().BackgroundBlockWall(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockChain(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockDChain(i,yCo)
					|| StageManager.GetInstance().BackgroundBlockTChain(i,yCo))
				{
					wallFlag = true;
					wallPos = i;
					//Debug.Log("FallOneLevel   find wall  [i="+i+",y="+yCo+"]");
					continue;
				}
				if(_data[i,yCo]>=1 && _data[i,yCo]<=_cubeTypeCount && _cubeData[i,yCo]!=null)
				{
					//script = _cubeData[i,yCo].GetComponent(typeof(GameCube)) as GameCube;
					//script.FallOneLevel();
					//_data[script.GetCOX(), script.GetCOY()] = 0;
				}
			}
		}
		foreach(GameObject obj in allCubes)
		{
			script = obj.GetComponent(typeof(GameCube)) as GameCube;
			if(script.GetCOY() == yCo && script.GetCOX()>xCo)
			{
				if(wallFlag)
				{
					if(script.GetCOX()<=wallPos 
						&& !StageManager.GetInstance().BackgroundBlockChain(script.GetCOX(),script.GetCOY())
						&& !StageManager.GetInstance().BackgroundBlockDChain(script.GetCOX(),script.GetCOY())
						&& !StageManager.GetInstance().BackgroundBlockTChain(script.GetCOX(),script.GetCOY()))
					{
						_data[script.GetCOX(), script.GetCOY()] = 0;
						script.FallOneLevel();
						//Debug.Log("FallOneLevel  [x="+xCo+",y="+yCo+"]    DO wallPos = "+wallPos);
					}
				}else
				{
					_data[script.GetCOX(), script.GetCOY()] = 0;
					script.FallOneLevel();
					//Debug.Log("FallOneLevel  [x="+xCo+",y="+yCo+"]    DO");
				}
					
			}
		}
		if(!noBirth)
		{
			GameObject cube;
			GameCube cubeScript;
			int cubeType;
			int count = 0;
			
			for(int j=row-1; j>=0; j--)
			{
				if(StageManager.GetInstance().BackgroundBlockWall(j,yCo)||
					CubeChain(j,yCo) ||
					CubeDChain(j,yCo) ||
					CubeTChain(j,yCo))
				{
					count++;
				}else{
					break;
				}
			}
			
			/**
			 * birth a cube
			 * */
			//Debug.Log("birth a cube   wallPos="+wallPos);
			//Debug.Log("birth a cube   yCo="+yCo);
			if(allCubes.Count < (row*colum+StageManager.GetInstance().WallBlockNum))
			{
				cubeType = GenerateRandomType();
				cube = Instantiate(cubePrefab) as GameObject;
				cube.transform.parent = _gamePanel.transform;
				cube.transform.localScale = new Vector3 (1, 1, 1);
				//SetBirthPlace(cube, row-1, colum);
				cube.transform.localPosition = new Vector3(
					(yCo+1)*tileWidth-tileWidth/2, 
					BIRTH_PLACE_OFFSET+fallingCubes.Count*tileWidth, 
					0
					);
				cubeScript = cube.GetComponent(typeof(GameCube)) as GameCube;
				//Debug.Log("FallOneLevel birth one cube fall N levels   N="+count);
				cubeScript.SetGameCoordinate(row-1-count,yCo);
				cubeScript.StartMove();
				cubeScript.SetCubeType(cubeType);
				
				AddFalling(cube);
				//Debug.Log("FallOneLevel  birth a cubebbbbbbb  [x="+(row-1-count)+",y="+yCo+"]    ");
				
				allCubes.Add(cube);
			}
		}
	}
	
	public void Hint()
	{
		GameHint.GetInstance().Show(hintX,hintY,hintDirection);
	}
	
	public bool ISStatic(int xCo, int yCo)
	{
		if(xCo<0 || xCo>=row)
			return false;
		
		if(yCo<0 || yCo>=colum)
			return false;
		
		if(_cubeData[xCo,yCo]!=null)
		{
			GameCube script;
			script = _cubeData[xCo,yCo].GetComponent(typeof(GameCube)) as GameCube;
			if(!script.IsMoving())
			{
				return true;
			}
		}
		
		return false;
	}
	
	public bool ISDestroying(int xCo, int yCo)
	{
		if(xCo<0 || xCo>=row)
			return false;
		
		if(yCo<0 || yCo>=colum)
			return false;
		
		if(_cubeData[xCo,yCo]!=null)
		{
			GameCube script;
			script = _cubeData[xCo,yCo].GetComponent(typeof(GameCube)) as GameCube;
			if(script.IsDestroying())
			{
				return true;
			}
		}
		
		return _data[xCo,yCo]==GAME_CUBE_DESTROYING;
	}
	
	public void GamePause()
	{
		_gamePause = true;
		if(timeMode)
		{
			backupPauseTime = System.DateTime.Now;
		}
	}
	
	public void GameResume()
	{
		if(timeMode)
		{
			System.DateTime now = System.DateTime.Now;
			_gameStartTime = _gameStartTime.AddMilliseconds(now.Subtract(backupPauseTime).TotalMilliseconds);
		}
		_gamePause = false;
		
		//MusicManager.GetInstance().PlayBGM(MusicManager.BGM_CLIP_ELECTRIC_ROMEO, 10);
	}
	
	public bool IsGamePause()
	{
		return _gamePause;
	}
	
	public void GameDestroy()
	{
		_gamePause = false;
		_gamePlaying = false;
	
		//MusicManager.GetInstance().StopBGM();
		
		GameCube script;
		
		/*
		foreach(GameObject obj in allCubes)
		{
			if(obj == null)
				continue;
			
			script = obj.GetComponent(typeof(GameCube)) as GameCube;
			if(script != null)
				script.DestroyMeIm();
		}
		*/
		int cubeCount = _gamePanel.transform.childCount;
		GameObject obj;
		for(int i=0; i<cubeCount; i++)
		{
			obj = _gamePanel.transform.GetChild(i).gameObject;
			
			if(obj == null)
				continue;
			
			script = obj.GetComponent(typeof(GameCube)) as GameCube;
			if(script != null)
				script.DestroyMeIm();
		}

		if(_cubeBackground != null)
		{
			Destroy(_cubeBackground);
		}
		
		DestroyGameBackground();
	}
	
	public void GameDestroyDelay(int frames)
	{
	}
	
	public int GetCubeDamage()
	{
		return Random.Range(CUBE_DAMAGE-3, CUBE_DAMAGE+3);
	}
	
	/**
	 * purchase callback
	 * */
	public void PurchaseCorns(string result)
	{
		Debug.Log("PurchaseCorns   callback result="+result);
		if(result.Equals("2000C"))
		{
			UserManager.GetInstance().AddCorns(2000);
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("5000C"))
		{
			UserManager.GetInstance().AddCorns(4400);
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("12000C"))
		{
			UserManager.GetInstance().AddCorns(7200);
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("21000C"))
		{
			UserManager.GetInstance().AddCorns(10400);
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("32000C"))
		{
			UserManager.GetInstance().AddCorns(15000);
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("45000C"))
		{
			UserManager.GetInstance().AddCorns(36666);
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("99999C"))
		{
			UserManager.GetInstance().AddCorns(66666);
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("IADRemove"))
		{
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("2MIN"))
		{
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			UserManager.GetInstance().Plus2Min = true;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("5MIN"))
		{
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			UserManager.GetInstance().Plus5Min = true;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("FULL"))
		{
			MessageWindow.GetInstance().ShowWarning("ɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			UserManager.GetInstance().Plus5Min = true;
			AdBannerManager.GetInstance().DisableAd();
			UserManager.GetInstance().AddCorns(99999);
			return;
		}
		if(result.Equals("FAIL"))
		{
			MessageWindow.GetInstance().ShowWarning("ʧܣ");
			IAPConnection.GetInstance().Purchasing = false;
			return;
		}
		if(result.Equals("RESTORE-F"))
		{
			MessageWindow.GetInstance().ShowWarning("ָʧܣ");
			IAPConnection.GetInstance().Purchasing = false;
			return;
		}
		if(result.Equals("RESTORE-I"))
		{
			MessageWindow.GetInstance().ShowWarning("ָɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			AdBannerManager.GetInstance().DisableAd();
			return;
		}
		if(result.Equals("RESTORE-2M"))
		{
			MessageWindow.GetInstance().ShowWarning("ָɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			UserManager.GetInstance().Plus2Min = true;
			return;
		}
		if(result.Equals("RESTORE-MM"))
		{
			MessageWindow.GetInstance().ShowWarning("ָɹףϷ죡");
			IAPConnection.GetInstance().Purchasing = false;
			UserManager.GetInstance().Plus5Min = true;
			return;
		}
	}
	
	bool chuiMode = false;
	int chuiX = -1;
	int chuiY = -1;
	public bool ChuiMode
	{
		get{
			return chuiMode;
		}
		set{
			chuiMode = value;
			if(!chuiMode)
			{
				ChuiScript.GetInstance().transform.localPosition = new Vector3(0,2000,-2);
				chuiX = chuiY = -1;
			}
		}
	}
	public int ChuiX
	{
		get{
			return chuiX;
		}
		set{
			chuiX = value;
		}
	}
	public int ChuiY
	{
		get{
			return chuiY;
		}
		set{
			chuiY = value;
		}
	}
	
}